///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXLog/LogDestination.h,v 1.2 2004/09/07 06:06:05 me133 Exp $
//
//    File name:	LogDestination.h
//   Created By:	Marc Eaddy
//   Created On:	10/03/2000 1:23:35 PM
//
//      Purpose:	Provides a destination for a log message
//
//  Description:	Base class that defines the log destination
//					interface and partial implementation for derived
//					classes.
//
//					Log destination objects are persisted to XML as
//					follows:
//
//					<DEST TYPE="type" [attributes]>
//
//					Here is an example:
//
//					<dest TYPE="LogFile" NAME="InfiniteLogFile"
//						FILE="ActiveILX.log" SAVE_BACKUP="true"/>
//
//					The only attribute supported by this base class
//					is TYPE.  Support for other attributes must be
//					provided by the derived classes.
//
//					In order for a log message to be routed to a
//					particular destination, the destination must have
//					a <DEST> element, and the <ROUTE> element's DEST
//					attribute must match the destination's TYPE
//					attribute.
//
//					For example, a route for the above destination
//					could be:
//
//					<route SEVERITY="INFO,SUCCESS" DEST="InfiniteLogFile"/>
//
//	          Note: If you add a new subclass, you must "register" it
//					in the CILXLog constructor by adding a new child
//					child prototype.
//
// Known
// Implementations: CLogFileDestination, CMsgBoxDestination,
//					CBeepDestination, CDebugOutDestination
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: LogDestination.h,v $
 * Revision 1.2  2004/09/07 06:06:05  me133
 * o Now uses most recent ILXLog code with ILXCrashFinder
 * 
 * 
 * 12    7/17/02 3:10p Marce
 * o Added preliminary support for passing exception info via the
 * SIlxStackFrame structure used by IlxCrashFinder
 * 
 * 9     11/10/00 1:25p Marce
 * o Comment
 * 
 * 7     10/25/00 4:16p Marce
 * o Comment
 * 
 * 6     10/25/00 1:37p Marce
 * o Added comments
 * 
 * 5     10/10/00 6:46p Marce
 * o Now passes the full path as well as the file name
 * o Fixed bug where trace messages were not clickable
 * 
 * 4     10/05/00 1:35p Marce
 * o Now provides a default implementation of initialize()
 * 
 * 3     10/04/00 5:38p Marce
 * o Implemented log filtering
 * 
 * 2     10/03/00 6:33p Marce
 * o Added log configuration classes
 */

#ifndef INCLUDED_LOG_DESTINATION_H
#define INCLUDED_LOG_DESTINATION_H

#include "XmlElement.h"	// Parent

// Forward declarations
struct SIlxStackFrames;

//*****************************
// CLogDestination
//*****************************

class CLogDestination : public CXmlElement
{
public:

	CLogDestination(const TCHAR* szSubType);
		//
		// PARAMETERS:
		//
		//	szSubType - Identifies the type of log destination.
		//		This is basically a mechanism for matching the
		//		TYPE attribute with a CLogDestination prototype
		//		for cloning (e.g., "MsgBox" matches
		//		CMsgBoxDestination.
		//

	virtual ~CLogDestination();

public:

	virtual HRESULT initialize();
		//
		// Called after CXmlElement::load() to allow the object
		// to extract the XML attributes needed to initialize
		// itself.  Must be called before log().
		//

	virtual HRESULT log(const TCHAR* szSubsystem, 
						const TCHAR* szPath,
						const TCHAR* szFilename, 
						long nLine, 
						long nThreadId, 
						long hrErrorCode,
						SIlxStackFrames* pvIlxStackFrames,
						long nNumStackFrames,
						const TCHAR* szMessage, 
						long nSeverity) = 0;
		//
		// Logs a message.  Must be overridden by the subclass
		// to send the message to a log file, message box, etc.
		//
		// PARAMETERS:
		//
		//	szSubsystem - (see ILXLogHelper.h)
		//
		//	szPath - The full path to the source file where the
		//		message was logged. (see ILXLogHelper.h)
		//
		//	szFilename - The file name of the source file where the
		//		message was logged.
		//
		//	nLine - The line number in the source file where the
		//		message was logged. (see ILXLogHelper.h)
		//
		//	nThreadId - The thread id of the thread that is logging
		//		the message.
		//
		//	hrErrorCode - The error/success code of the message if
		//		any.  -1 means that no code is associated with the
		//		log message.  The code is only for diagnostic purposes
		//		because the error message for the code has already
		//		been found and is specified in szMessage.
		//
		//	szMessage - The message to be logged.
		//
		//	nSeverity - One of the ILXLOG_SEVERITIES enumeration
		//		values (see ILXLog.idl).  You will need to include
		//		ILXLog2.h to use the enum.
		//
		// RETURNS:	S_OK on success, otherwise a failure code
		//

	bool isInitialized() const;
		//
		// TRUE if initialize() has been called for this object,
		// otherwise FALSE.
		//

protected:
	// Overrides CXmlElement
	virtual CXmlElement* clone() const = 0;
		//
		// Clones this element.
		//

private: // NOT IMPLEMENTED
	CLogDestination(const CLogDestination&);
	CLogDestination& operator=(const CLogDestination&);

protected: // DATA
	bool	m_bIsInitialized; // Has initialize() been called yet?
};

#endif // INCLUDED_LOG_DESTINATION_H