///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXLog/LogFileDestination.h,v 1.2 2004/09/07 06:06:05 me133 Exp $
//
//    File name:	LogFileDestination.h
//   Created By:	Marc Eaddy
//   Created On:	10/03/2000 1:33:01 PM
//
//      Purpose:	Outputs log messages to a log file
//
//  Description:	Creates messages that look like
//
//					10/25 13:29:25 362 - INFO - svcwrp: <GDM123_67> CCtrlFrame::
//						OnDestroy(). [File: CtrlFrame.cpp, Line: 301]
//					10/25 13:30:27  47 - ERROR - svcwrp: <GDM123_67> Timeout
//						expired. - A service was terminated after failing to
//						shutdown.  (0xC4010011) [File: ThreadControlHelper.cpp,
//						Line: 580]
//
//   Xml Format:
//					Example:
//
//					<DEST TYPE="LogFile" NAME="ErrorFile" NAME="Error.log"
//					 SAVE_BACKUP="true" MAX_SIZE="123456"/>
//                                                           
//					Attributes:                                            
//                                                           
//					NAME - A logical name for the log file destination.
//						This allows you have two different log file destinations
//						each with different settings. Defaults to TYPE.
//						You must specify a NAME if you have two DEST elements
//						with the same type.
//					                                                         
//					FILE - The file name of the log file.  The log file will
//						always be stored in the RDC Log File directory.
//					                                                         
//					SAVE_BACKUP - Whether or not to backup the log file before
//						overwriting it.  The backup file will be named FILE.bak
//						(the FILE attribute).  The default is "true".
//					                                                         
//					MAX_SIZE - The maximum size of the log file.  If the size
//						is exceeded the log file will be deleted and then
//						recreated.  If SAVE_BACKUP is true then the file will
//						be backed up before it is deleted.
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: LogFileDestination.h,v $
 * Revision 1.2  2004/09/07 06:06:05  me133
 * o Now uses most recent ILXLog code with ILXCrashFinder
 * 
 * 
 * 11    7/17/02 3:10p Marce
 * o Added preliminary support for passing exception info via the
 * SIlxStackFrame structure used by IlxCrashFinder
 * 
 * 7     10/25/00 3:56p Marce
 * o Removed critical section because it is redundant.  A critical section
 * is already maintained by ILXLogHelper.cpp.  However, this might be a
 * problem if we use IILXLog directly (e.g., from scripting) since no
 * marshalling occurs and multiple threads can log messages at the same
 * time.  However, we should handle this in the general case not just for
 * CLogFileDestination.  I don't think this will be a problem for now
 * because scripting only logs messages rarely.
 * 
 * 6     10/25/00 3:10p Marce
 * o Comments
 * 
 * 5     10/25/00 2:54p Marce
 * o Comments
 * 
 * 4     10/10/00 6:46p Marce
 * o Now passes the full path as well as the file name
 * o Fixed bug where trace messages were not clickable
 * 
 * 3     10/04/00 5:38p Marce
 * o Implemented log filtering
 * 
 * 2     10/03/00 6:33p Marce
 * o Added log configuration classes
 */

#ifndef INCLUDED_LOG_FILE_DESTINATION_H
#define INCLUDED_LOG_FILE_DESTINATION_H

#include "LogDestination.h"	// Parent

// Type defines

typedef std::vector<long>				TypeVecLongs;

//*****************************
// CLogFileDestination
//*****************************

class CLogFileDestination : public CLogDestination
{
public:

	CLogFileDestination();
	virtual ~CLogFileDestination();

public:

	virtual HRESULT initialize();
		//
		// See LogDestination.h for a description
		//

	virtual HRESULT log(const TCHAR* szSubsystem, 
						const TCHAR* szPath,
						const TCHAR* szFilename, 
						long nLine, 
						long nThreadId, 
						long hrErrorCode,
						SIlxStackFrames* pvIlxStackFrames,
						long nNumStackFrames,
						const TCHAR* szMessage, 
						long nSeverity);
		//
		// See LogDestination.h for a description.
		//

	void getLogHistory(/*[out]*/ tstring* pstrHistory);
		//
		// Returns the last 20 lines of the log file.
		// 

	void setLogFile(const TCHAR* szFilename);
		//
		// Sets the log filename manually.  Must be called before 
		// initialize().  Usually, you don't set this manually since
		// it gets set when you load from XML.
		//

	void setMaxSize(long nMaxSize);
		//
		// Sets the max log file size manually.  Must be called before
		// initialize().  Usually, you don't set this manually since
		// it gets set when you load from XML.
		//

protected:
	// Overrides CXmlElement
	virtual CXmlElement* clone() const;
		//
		// Clones this element.
		//

private: // HELPER FUNCTIONS

	HRESULT backupFile();
		//
		// Backups up the logfile.
		// 

	HRESULT createLogFile(tstring* pstrBackupFilename = 0);
		//
		// Creates the log file and outputs the log file header.
		//

	HRESULT logNoBackup(	const TCHAR* szSubsystem, 
							const TCHAR* szFilename, 
							long nLine, 
						    long nThreadId, 
							const TCHAR* szMessage, 
							long nSeverity);
		//
		// Actually logs the message.
		//

private: // NOT IMPLEMENTED

	CLogFileDestination(const CLogFileDestination&);
	CLogFileDestination& operator=(const CLogFileDestination&);

private:	// DATA
	FILE*				m_fileLog;				
		// Log file handle

	long				m_nMaxFileSize;			
		// Max size of log file

	long				m_nLogLineCount;
		// Current line number in log file

	TypeVecLongs		m_vecLogHistorySeekPos;
		// Seek positions of the last 20 log file lines.  Use to
		// implement getLogHistory().

	tstring				m_strLogFile;
		// Name of the log file

	SYSTEMTIME			m_stStartTime;
		// Log file creation time

	bool				m_bSaveBackup;
		// Whether or not we backup the log file
};

#endif // INCLUDED_LOG_FILE_DESTINATION_H