///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXLog/LogRoute.cpp,v 1.2 2004/09/07 06:06:05 me133 Exp $
//
//    File name:	LogRoute.cpp
//   Created By:	Marc Eaddy
//   Created On:	10/03/2000 2:52:18 PM
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: LogRoute.cpp,v $
 * Revision 1.2  2004/09/07 06:06:05  me133
 * o Now uses most recent ILXLog code with ILXCrashFinder
 * 
 * 
 * 10    1/30/01 10:51a Marce
 * o Fixed compile errors for Debug build
 * 
 * 9     11/10/00 4:23p Marce
 * o Added comments
 * 
 * 7     11/08/00 1:40p Marce
 * o Added comments
 * o Removed duplicated m_bLoaded boolean (base class handles this)
 * 
 * 6     10/05/00 2:59p Marce
 * o Was checking the wrong collection
 * 
 * 5     10/05/00 2:51p Marce
 * o Now asserts if no destinations are specified
 * 
 * 4     10/05/00 2:41p Marce
 * o Collapsed SUBSYSTEM and FILE into SOURCE
 * 
 * 3     10/04/00 5:38p Marce
 * o Implemented log filtering
 * 
 * 2     10/03/00 6:33p Marce
 * o Added log configuration classes
 */

#include "StdAfx.h"

#include "LogRoute.h"		// Me

#include "StringParsing.h"	// DelimitedString2StringList()

#include "IlxLog2.h"		// ILXLOG__ERROR, etc.

#include "LogDestination.h"	// CLogDestination

#include <vector>			// STL vector<>
#include <algorithm>		// STL find()
#include <functional>		// STL less<>

// Constants

static const TCHAR* const szSOURCE_ATTR			= _T("SOURCE");
//static const TCHAR* const szFACILITY_ATTR		= _T("FACILITY"); // Not used
static const TCHAR* const szSEVERITY_ATTR		= _T("SEVERITY");
static const TCHAR* const szDESTINATION_ATTR	= _T("DEST");
static const TCHAR* const szCODE_ATTR			= _T("CODE");

static const TCHAR cDELIMITER					= _T(',');

// HELPER FUNCTIONS
static void SeverityAttr2NumList(const TCHAR* szBuffer, TypeListNums* plistSeverities);

/////////////////////////////
// LOG ROUTE IMPLEMENTATION
/////////////////////////////

// Initialize static members

const tstring CLogRoute::s_strXML_TAG_NAME		= _T("ROUTE");

CLogRoute::CLogRoute()
	: CXmlElement(s_strXML_TAG_NAME.c_str(), _T(""), _T(""), false)
	, m_listSources()
	, m_listSeverities()
	, m_listErrorCodes()
	, m_setDestinations()
{ }

CLogRoute::~CLogRoute()
{ }

CXmlElement* CLogRoute::clone() const
{
	return new CLogRoute();
}

HRESULT CLogRoute::load(MSXML::IXMLDOMNode* piNode)
{
	HRESULT hr = S_OK;

	if (!piNode) return E_POINTER;

	// Invariants
	_ASSERTE(m_listSources.size() == 0);
	_ASSERTE(m_listSeverities.size() == 0);
	_ASSERTE(m_listErrorCodes.size() == 0);
	_ASSERTE(m_setDestinations.size() == 0);

	hr = CXmlElement::load(piNode);
//	VERIFY_HR_ASSERT(hr, _T("Failed to load log XML entry."), ILXLOG_ERROR); 
	_ASSERTE(SUCCEEDED(hr));
	if (FAILED(hr)) return hr;

	USES_CONVERSION;

	// Step 1:	Initialize member variables

	// Parse the SOURCE attribute
	const _variant_t& varSources = m_mapAttributes[szSOURCE_ATTR];
	if (varSources.vt == VT_BSTR && varSources.bstrVal)
	{
		::DelimitedString2StringList(	OLE2CT(varSources.bstrVal),
										&m_listSources,
										cDELIMITER);
	}

	// Parse the SEVERITY attribute
	const _variant_t& varSeverities = m_mapAttributes[szSEVERITY_ATTR];
	if (varSeverities.vt == VT_BSTR && varSeverities.bstrVal)
	{
		SeverityAttr2NumList(	OLE2CT(varSeverities.bstrVal),
								&m_listSeverities);
	}

	// Parse the CODE attribute
	const _variant_t& varErrorCodes = m_mapAttributes[szCODE_ATTR];
	if (varErrorCodes.vt == VT_BSTR && varErrorCodes.bstrVal)
	{
		::DelimitedString2NumList(	OLE2CT(varErrorCodes.bstrVal),
									&m_listErrorCodes,
									cDELIMITER);
	}

//	_bstr_t bstrDestinations	= static_cast<_bstr_t>(m_mapAttributes[szDESTINATION_ATTR]);

	return S_OK;
}

//*********************************************************************
// FUNCTION		:	CLogRoute::initDestinations
//					
// DESCRIPTION	:	Extracts the destination pointers for this route
//					from the master destination list and initialize
//					them.
//					
// PARAMETERS	:	
//   plistDestinations	
//					
//*********************************************************************
void CLogRoute::initDestinations(TypeSetDestinations* plistDestinations)
{
	USES_CONVERSION;

	// Verify arguments
	_ASSERTE(plistDestinations);

	// Invariants
	_ASSERTE(m_setDestinations.size() == 0);
	_ASSERTE(isLoaded());

	// Parse the DEST attribute
	const _variant_t& varDestinations = m_mapAttributes[szDESTINATION_ATTR];
	if (varDestinations.vt != VT_BSTR || !varDestinations.bstrVal)
	{
		return;
	}

	TypeListStrings listDestinationStrs;

	::DelimitedString2StringList(	OLE2CT(varDestinations.bstrVal),
									&listDestinationStrs,
									cDELIMITER);
	
	// Invariant: At least one destination per route since we are using
	// any matching route not just the best route.
	_ASSERTE(listDestinationStrs.size() != 0);

	bool bFound = false;

	// Step 1:	Iterate through all the destinations in the master list

	for(TypeListStrings::iterator it = listDestinationStrs.begin();
		it != listDestinationStrs.end();
		++it)
	{
		const tstring& strDestination = *it;
		_ASSERTE(strDestination.length() > 0);

		bFound = false;

		// Step 2:	See if the destination belongs to this route

		for(TypeSetDestinations::iterator it_dest = plistDestinations->begin();
			it_dest != plistDestinations->end();
			++it_dest)
		{
			CLogDestination* pLogDestination = *it_dest;
			_ASSERTE(pLogDestination);

			const OLECHAR* szLogDestinationName = pLogDestination->getNameAttr();
			const tstring& strLogDestinationType = pLogDestination->getSubType();

			if (szLogDestinationName)
			{
				if (wcsicmp(pLogDestination->getNameAttr(), T2CW(strDestination.c_str())) == 0)
				{
					bFound = true; // The destination belongs to this route!
				}
			}
			else if (strLogDestinationType == strDestination)
			{
				bFound = true; // The destination belongs to this route!
			}

			// The destination belongs to this route so initialize it
			// and add it to this route's destination set
			if (bFound)
			{
				if (!pLogDestination->isInitialized())
				{
					pLogDestination->initialize();
				}
				
				m_setDestinations.insert(pLogDestination);
				break;
			}
		}

		_ASSERTE(bFound || !"Destination not found");
	}
}

//*********************************************************************
// FUNCTION		:	CLogRoute::matches
//					
// DESCRIPTION	:	Determines if the message matches this route.
//					
// RETURNS		:	OR'd values from the EMatches enumeration.
//   long 			
//					
// PARAMETERS	:	
//   szSubsystem	
//   szFile	
//   nLine	
//   nThreadId	
//   hrErrorCode	
//   szMessage	
//   nSeverity	
//					
// COMMENTS		:	
//					
//*********************************************************************
long CLogRoute::matches(	const TCHAR* szSubsystem, 
							const TCHAR* szFile, 
							long nLine, 
							long nThreadId, 
							long hrErrorCode, 
							const TCHAR* szMessage, 
							long nSeverity) const
{
	long nMatchResult = MATCHES_NONE;

	// The route's SOURCE attribute can match either the file or subsystem.
	// These are usually mutually exclusive since a file ends with .cpp, .c,
	// or .h and a subsystem has no extension.
	if (std::find(m_listSources.begin(), m_listSources.end(), szSubsystem) 
		!= m_listSources.end() ||
		std::find(m_listSources.begin(), m_listSources.end(), szFile) 
		!= m_listSources.end())
	{
		nMatchResult |= MATCHES_SOURCE;
	}

	if (std::find(m_listErrorCodes.begin(), m_listErrorCodes.end(), hrErrorCode) 
		!= m_listErrorCodes.end())
	{
		nMatchResult |= MATCHES_ERROR_CODE;
	}

	if (std::find(m_listSeverities.begin(), m_listSeverities.end(), nSeverity) 
		!= m_listSeverities.end())
	{
		nMatchResult |= MATCHES_SEVERITY;
	}

	// Sanity checks: Can't match both the CODE and SEVERITY attributes
	// since the CODE precludes the SEVERITY.  This would happen if you
	// specified a CODE whose severity matches SEVERITY such as
	// CODE="0x80004005" SEVERITY="ERROR".  However, its not the end of
	// the world!
	_ASSERTE(!(nMatchResult & MATCHES_ERROR_CODE && nMatchResult & MATCHES_SEVERITY));

	return nMatchResult;
}

TypeSetDestinations& CLogRoute::getDestinations()
{
	return m_setDestinations;
}

// HELPER FUNCTIONS

//*********************************************************************
// FUNCTION		:	SeverityAttr2NumList
//					
// DESCRIPTION	:	Converts a comma-delimited list of severities into
//					a list of ILXLOG_SEVERITIES enum values.
//					
// PARAMETERS	:	
//   szBuffer			Ex: "ERROR,SUCCESS"
//   plistSeverities
//					
//*********************************************************************
void SeverityAttr2NumList(const TCHAR* szBuffer, TypeListNums* plistSeverities)
{
	TypeListStrings listSeverities;
	::DelimitedString2StringList(szBuffer, &listSeverities, cDELIMITER);
	
	for(TypeListStrings::iterator it = listSeverities.begin();
		it != listSeverities.end();
		++it)
	{
		const tstring& strSeverity = *it;
		_ASSERTE(strSeverity.length() > 0);
		
		if (strSeverity == _T("ERROR"))
		{
			plistSeverities->push_back(ILXLOG__ERROR);
		}
		else if (strSeverity == _T("WARNING"))
		{
			plistSeverities->push_back(ILXLOG__WARNING);
		}
		else if (strSeverity == _T("INFO"))
		{
			plistSeverities->push_back(ILXLOG__INFO);
		}
		else if (strSeverity == _T("SUCCESS"))
		{
			plistSeverities->push_back(ILXLOG__SUCCESS);
		}
		else
		{
			_ASSERTE(!"Unrecognized severity!");
		}
	}
}
