///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/ILXLog/LogRoute.h,v 1.2 2004/09/07 06:06:05 me133 Exp $
//
//    File name:	LogRoute.h
//   Created By:	Marc Eaddy
//   Created On:	10/03/2000 2:31:14 PM
//
//      Purpose:	Routes log messages to zero or more log destinations
//
//  Description:	Provides a log message mask/filter that matches
//					a type of message (e.g., errors) or a specific
//					message (e.g., 0x80004005), and provides a list
//					of log destinations for that message.
//
//					Log route objects are persisted to XML as follows:
//
//					<ROUTE SEVERITY="severity" SOURCE="source" DEST="dest/>
//					or
//					<ROUTE CODE="severity" SOURCE="source" DEST="dest/>
//
//					Here is an example:
//
//					<route SEVERITY="ERROR,WARNING"
//						DEST="Beep,InfiniteLogFile,DebugOut"/>
//
//					The SEVERITY, CODE, and SOURCE attributes can have
//					multiple values separated by commas.  SEVERITY
//					must be one of "ERROR", "WARNING", "INFO", or
//					"SUCCESS".  CODE can be any hexadecimal HRESULT
//					value or number (e.g., "0x80004005", "13").
//					SOURCE is the exact source file name or subsystem
//					where the message was logged.
//
//					The DEST attribute is a comma-delimited list of log
//					destinations.  The valid log destinations must be
//					previously specified using the <DEST> element (see
//					LogDestination.h).
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: LogRoute.h,v $
 * Revision 1.2  2004/09/07 06:06:05  me133
 * o Now uses most recent ILXLog code with ILXCrashFinder
 * 
 * 
 * 4     11/08/00 1:40p Marce
 * o Added comments
 * o Removed duplicated m_bLoaded boolean (base class handles this)
 * 
 * 3     10/05/00 2:41p Marce
 * o Collapsed SUBSYSTEM and FILE into SOURCE
 * 
 * 2     10/04/00 5:38p Marce
 * o Implemented log filtering
 */

#ifndef INCLUDED_LOG_ROUTE_H
#define INCLUDED_LOG_ROUTE_H

#include "XmlElement.h"	// Parent

#include <set>			// STL set<>
#include <list>			// STL list<>

// Forward declarations
class CLogDestination;

// Type defines

typedef std::list<long>				TypeListNums;
typedef std::set<CLogDestination*>	TypeSetDestinations;
typedef std::list<tstring>			TypeListStrings;

enum EMatches 
{
	MATCHES_NONE		= 0,
	MATCHES_SOURCE		= 1,
	MATCHES_SEVERITY	= 2,
	MATCHES_ERROR_CODE	= 4,
//	MATCHES_FACILITY	= 8,	// Not used
};

//*****************************
// CLogRoute
//*****************************

class CLogRoute : public CXmlElement
{
public:

	CLogRoute();
	virtual ~CLogRoute();

public:
	// Overrides CXmlElement
	virtual HRESULT load(MSXML::IXMLDOMNode* piNode);
		//
		// Loads the element attributes from the element node.
		//
		// PARAMETERS:
		//
		//	spiElement - This element MUST have its NAME attribute equal
		//		to this layout elements tag name.  For example, if the
		//		element has the name "EVENT_TABLE", then the only
		//		subclass that can load from the element is CEventTable.
		//

	void initDestinations(TypeSetDestinations* plistDestinations);
		//
		// Allows the log route to extract the subset of log
		// destinations from the master list based on the ones
		// listed in the DEST attribute.
		//
		// MUST be called after load() but before matches() or
		// getDestinations().
		//

	long matches(	const TCHAR* szSubsystem, 
					const TCHAR* szFile, 
					long nLine, 
					long nThreadId, 
					long hrErrorCode, 
					const TCHAR* szMessage, 
					long nSeverity) const;
		//
		// Determines if the message should be routed by
		// this log route.
		//
		// RETURNS: Values of the EMatches enumeration that can be 
		//	OR'd together.
		//

	TypeSetDestinations& getDestinations();
		//
		// Returns the set of log destinations associated with
		// this log route.  This is a subset of the master list
		// of log destinations and is specified by the DEST
		// attribute.
		//

protected:
	// Overrides CXmlElement
	virtual CXmlElement* clone() const;
		//
		// Clones this element.
		//

private: // NOT IMPLEMENTED

	CLogRoute(const CLogRoute&);
	CLogRoute& operator=(const CLogRoute&);

private: // DATA

	TypeListStrings		m_listSources;		// Sources handled by this route
	TypeListNums		m_listSeverities;	// Severities handled by this route
	TypeListNums		m_listErrorCodes;	// Error codes handled by this route
	TypeSetDestinations	m_setDestinations;	// Log destinations for this route

public:

	static const tstring s_strXML_TAG_NAME;
};

#endif // INCLUDED_LOG_ROUTE_H