///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /Common/ILXLog/ILXLogHelper.h 3     12/09/05 1:28p Eaddy $
//
//    File name:	ILXLogHelper.h
//   Created By:	Marc Eaddy
//   Created on:	04/24/2000 12:11:30 PM
//
//      Purpose:	Singleton for logging error messages
//
//  Description:	This class provides global access to a singleton
//					object for logging error messages.  The singleton
//					provides these benefits:
//
//					o Clients do not need to store any extra pointers,
//					  they simply use the ILXLOG macros.
//					o Can be called from any thread, COM apartment or DLL
//
//					Additional benefits are:
//
//					o Log destination transparency (log file, NT Event
//					  Log, message box, email, etc.)
//					o Converts HRESULTs to string messages
//					o Localization of messages
//
//					Here is the format for a log message:
//
//					<DATE> - <SEVERITY> - <SUBSYSTEM> - <MESSAGE> [FILE, LINE]
//
//					Here is what a typical log message looks like
//					(reformatted to fit):
//
//					4/25 12:07:10 - ERROR - ActiveILX: Failed to register
//						workstation object in the running object table. - 
//						Moniker is already registered in running object table 
//						(0x000401E7) [File: ILXWorkStation.cpp, Line: 816]
//
//		   Note:	See the CILXLogHelper2 class note for information on
//					initialization.
//					
//		  Usage:	Intended for use by any client C++ code that needs to
//					perform logging.
//
//					1) Include IlxLogHelper.h
//					2) Link with IlxLog.lib
//					3) Use the ILXLOG(), ILXLOGMSG(), etc. macros
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Common/ILXLog/ILXLogHelper.h $
 * 
 * 3     12/09/05 1:28p Eaddy
 * o Added macros that take 7 arguments
 * 
 * 2     12/09/05 1:21p Eaddy
 * o Added macros that take 6 arguments
 * Revision 1.2  2004/09/07 06:06:04  me133
 * o Now uses most recent ILXLog code with ILXCrashFinder
 * 
 * 
 * 38    7/22/02 11:39a Marce
 * o Argument should be const
 * 
 * 37    7/22/02 11:18a Marce
 * o ILXLOGCRASH macros now takes different arguments
 * 
 * 36    7/18/02 12:08a Zakirh
 * integration with crashfinder.
 * 
 * 35    7/17/02 3:10p Marce
 * o Added preliminary support for passing exception info via the
 * SIlxStackFrame structure used by IlxCrashFinder
 * 
 * 34    7/12/02 7:56p Marce
 * o Now most macros have a _, a _V and a CHECK version
 * 
 * 33    7/12/02 2:54p Marce
 * o Added VERIFY_HR_ macros that allow you to specify the return value to
 * use instead of hr.  VERIFY_HR_V macros don't return anything (void).
 * o Added CHECK_HR to allow you to log an error message if it happens but
 * not return
 * 
 * 30    4/09/01 11:22a Marce
 * o Now allows up to five parameters in the macros
 * 
 * 29    3/22/01 5:42p Marce
 * o Now allows up to four parameters in macros
 * 
 * 28    2/12/01 12:14p Marce
 * o Versioned the interface for Phase3 by calling it CILXLogHelper2
 * 
 * 27    1/30/01 12:08p Marce
 * o Added API versioning to prevent DLL mismatches that cause undefined
 * runtime behavior
 * o Add CoInit and CoUninit since we are creating COM objects. This was
 * breaking for clients that did not do this prior to using the log
 * helper.
 * 
 * 26    1/19/01 5:57p Marce
 * o Now most macros can take up to 3 parameters
 * 
 * 25    1/10/01 3:15p Marce
 * o Added VERIFY_HR3
 * 
 * 24    11/28/00 6:33p Marce
 * o Added VERIFY_COND2
 * 
 * 23    11/17/00 6:01p Marce
 * o addReourceModule() and addResouceFile() now return an HRESULT to
 * provide error info
 * o Add ILXLOG0 and other XXX0 macros to allow you to specify 0 arguments
 * and put severity before the message for consistency.
 * 
 * 22    11/15/00 2:33p Marce
 * o Removed the original LogHR().  Renamed LogHR2() to LogHR().
 * o Removed the overloaded log() method that did not use a severity
 * o Changed the severity param to use the enum for better type checking
 * o Now supports message formatting
 * o Added more comments
 * 
 * 20    10/25/00 3:57p Marce
 * o Added comments
 * 
 * 19    10/05/00 4:47p Marce
 * o Added comments
 * 
 * 18    10/05/00 1:47p Marce
 * o Clarified parameter names
 * 
 * 17    10/04/00 5:59p Marce
 * o Added MSXML forward declares to fix compile errors
 * 
 * 16    10/04/00 5:39p Marce
 * o Removed Initialize() and load()
 * o Added LoadConfig() and UseLogFile()
 * o Implemented log filtering
 * 
 * 15    6/02/00 6:05p Marce
 * o E_UNEXPECTED was a little too harsh
 * 
 * 14    5/31/00 4:47p Marce
 * o Added overloaded log() function that allows you to specify the
 * subsystem explicitly
 * 
 * 13    5/13/00 12:49a Marce
 * o Added the ability to pop-up a message box with a string message
 * 
 * 12    5/10/00 3:03p Marce
 * o Rearranged assert so  that the message is logged first
 * 
 * 11    5/05/00 1:59p Marce
 * o Now ILXLogHelper is part of ILXLog.dll
 * 
 * 10    5/04/00 1:58p Marce
 * o Now IILXLog aggregates the FTM so we don't need to perform
 * marshalling
 * 
 * 9     5/01/00 11:32a Marce
 * o Now logs source file and line number
 * 
 * 8     4/27/00 8:20p Marce
 * o Changed macro name
 * 
 * 7     4/25/00 9:51p Marce
 * o Now the message box only takes an HRESULT
 * 
 * 6     4/25/00 8:31p Marce
 * o Added support for including resource modules to search when resolving
 * HRESULTs
 * o Changed "Module" to "Subsystem" to avoid confusion with resource
 * modules
 * 
 * 5     4/25/00 2:50p Marce
 * o Added comments
 * 
 * 4     4/25/00 1:33p Marce
 * o Now uses 4 severities instead of 30 levels
 * o Separated out error dialog popup from normal logging
 * 
 * 3     4/24/00 4:09p Marce
 * o Fixed bugs caused by not marshalling the IILXLog object properly
 */

#ifndef INCLUDED_ILX_LOG_HELPER2_H
#define INCLUDED_ILX_LOG_HELPER2_H

#include <atlbase.h>	// CComBSTR
#include <comdef.h>	// _COM_SMARTPTR_TYPEDEF

#ifdef DLLPort
	#undef DLLPort
#endif // DLLPort

#ifdef ILXLOG_EXPORTS
	#define DLLPort	__declspec(dllexport)
#else
	#define DLLPort	__declspec(dllimport)
#endif // ILXLOG_EXPORTS

// ILX_LOG_API_VERSION will be updated when changes to IlxLog API
// require clients to recompile.  If a client was compiled
// against an older version of the API and they attempt to call
// a logging macro or function, they will see an error message box
// indicating that they need to recompile.  The actual version
// number is unimportant. (borrowed this trick from DirectX 8.0)
#define ILX_LOG_API_VERSION		111

//--------------------------------------------------------------
// Macros
//--------------------------------------------------------------
// The ones with a number suffix (1, 2, 3, or 4) can take that
// number of extra parameters to the printf-style format string
// specified by the "msg" parameter.
//--------------------------------------------------------------

// Log a message that has an associated HRESULT

#define ILXLOG(hr, msg, severity) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).log(__FILE__, __LINE__, hr, msg, severity)

#define ILXLOG0(hr, severity, msg) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg)

#define ILXLOG1(hr, severity, msg, arg1) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1)

#define ILXLOG2(hr, severity, msg, arg1, arg2) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2)

#define ILXLOG3(hr, severity, msg, arg1, arg2, arg3) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3)

#define ILXLOG4(hr, severity, msg, arg1, arg2, arg3, arg4) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4)

#define ILXLOG5(hr, severity, msg, arg1, arg2, arg3, arg4, arg5) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5)

#define ILXLOG6(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)

#define ILXLOG7(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)

// Log a message that has associated EXCEPTION INFO

#define ILXLOGCRASH( info, szSearchPath, severity, szFormat) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat)

#define ILXLOGCRASH0( info, szSearchPath, severity, szFormat) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat)

#define ILXLOGCRASH1( info, szSearchPath, severity, szFormat, arg1 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1 )

#define ILXLOGCRASH2( info, szSearchPath, severity, szFormat, arg1, arg2 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2 )

#define ILXLOGCRASH3( info, szSearchPath, severity, szFormat, arg1, arg2, arg3 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2, arg3 )

#define ILXLOGCRASH4( info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4 )

#define ILXLOGCRASH5( info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5 )

#define ILXLOGCRASH6( info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5, arg6 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5, arg6 )

#define ILXLOGCRASH7( info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5, arg6, arg7 ) \
	CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION).vlogCrash(__FILE__, __LINE__, info, szSearchPath, severity, szFormat, arg1, arg2, arg3, arg4, arg5, arg6, arg7 )

// Log a message that doesn't have an associated HRESULT
#define ILXLOGMSG(msg, severity) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->log(__FILE__, __LINE__, msg, severity);						\
	}

#define ILXLOGMSG0(severity, msg)													\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg);					\
	}

#define ILXLOGMSG1(severity, msg, arg1)												\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1);				\
	}

#define ILXLOGMSG2(severity, msg, arg1, arg2)										\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2);		\
	}

#define ILXLOGMSG3(severity, msg, arg1, arg2, arg3)									\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3);	\
	}

#define ILXLOGMSG4(severity, msg, arg1, arg2, arg3, arg4) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4);	\
	}

#define ILXLOGMSG5(severity, msg, arg1, arg2, arg3, arg4, arg5)						\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5);	\
	}

#define ILXLOGMSG6(severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)				\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6);	\
	}

#define ILXLOGMSG7(severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)			\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7);	\
	}

// Display message in a modal dialog box (in a separate thread)

#define ILXMSGBOX(hr)																\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->msgBox(__FILE__, __LINE__, hr);								\
	}

#define ILXMSGBOX0(severity, msg) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg);					\
	}

#define ILXMSGBOX1(severity, msg, arg1) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1);			\
	}

#define ILXMSGBOX2(severity, msg, arg1, arg2) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2);		\
	}

#define ILXMSGBOX3(severity, msg, arg1, arg2, arg3) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3); \
	}

#define ILXMSGBOX4(severity, msg, arg1, arg2, arg3, arg4) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4); \
	}

#define ILXMSGBOX5(severity, msg, arg1, arg2, arg3, arg4, arg5) \
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5); \
	}

#define ILXMSGBOX6(severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)				\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
	}

#define ILXMSGBOX7(severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)			\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vmsgBox(__FILE__, __LINE__, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
	}

// Macros for checking HRESULT return codes and returning the failed HRESULT

#define VERIFY_HR(hr, msg, severity)												\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->log(__FILE__, __LINE__, hr, msg, severity);			\
		else return E_FAIL;															\
	}

#define VERIFY_HR0(hr, severity, msg)												\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg);			\
		else return E_FAIL;															\
	}

#define VERIFY_HR1(hr, severity, msg, arg1)											\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1);	\
		else return E_FAIL;															\
	}

#define VERIFY_HR2(hr, severity, msg, arg1, arg2)									\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2); \
		else return E_FAIL;															\
	}

#define VERIFY_HR3(hr, severity, msg, arg1, arg2, arg3)								\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3); \
		else return E_FAIL;															\
	}

#define VERIFY_HR4(hr, severity, msg, arg1, arg2, arg3, arg4)						\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4); \
		else return E_FAIL;															\
	}

#define VERIFY_HR5(hr, severity, msg, arg1, arg2, arg3, arg4, arg5)					\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		else return E_FAIL;															\
	}

#define VERIFY_HR6(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)			\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		else return E_FAIL;															\
	}

#define VERIFY_HR7(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)		\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) return pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		else return E_FAIL;															\
	}

// Macros for checking HRESULT return codes and returning a specific value

#define VERIFY_HR_(ret, hr, msg, severity)											\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->log(__FILE__, __LINE__, hr, msg, severity);					\
		return ret;																	\
	}

#define VERIFY_HR0_(ret, hr, severity, msg)											\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg);				\
		return ret;																	\
	}

#define VERIFY_HR1_(ret, hr, severity, msg, arg1)									\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1);			\
		return ret;																	\
	}

#define VERIFY_HR2_(ret, hr, severity, msg, arg1, arg2)								\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2);	\
		return ret;																	\
	}

#define VERIFY_HR3_(ret, hr, severity, msg, arg1, arg2, arg3)						\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3); \
		return ret;																	\
	}

#define VERIFY_HR4_(ret, hr, severity, msg, arg1, arg2, arg3, arg4)					\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4); \
		return ret;																	\
	}

#define VERIFY_HR5_(ret, hr, severity, msg, arg1, arg2, arg3, arg4, arg5)			\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		return ret;																	\
	}

#define VERIFY_HR6_(ret, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)		\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		return ret;																	\
	}

#define VERIFY_HR7_(ret, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		return ret;																	\
	}

// Macros for checking HRESULT return codes and returning on failure

#define VERIFY_HR_V(hr, msg, severity)												\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->log(__FILE__, __LINE__, hr, msg, severity);					\
		return;																		\
	}

#define VERIFY_HR0_V(hr, severity, msg)												\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg);				\
		return;																		\
	}

#define VERIFY_HR1_V(hr, severity, msg, arg1)										\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1);			\
		return;																		\
	}

#define VERIFY_HR2_V(hr, severity, msg, arg1, arg2)									\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2);	\
		return ret;																	\
	}

#define VERIFY_HR3_V(hr, severity, msg, arg1, arg2, arg3)							\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3); \
		return;																		\
	}

#define VERIFY_HR4_V(hr, severity, msg, arg1, arg2, arg3, arg4)						\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4); \
		return;																		\
	}

#define VERIFY_HR5_V(hr, severity, msg, arg1, arg2, arg3, arg4, arg5)				\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		return;																		\
	}

#define VERIFY_HR6_V(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)			\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		return;																		\
	}

#define VERIFY_HR7_V(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		return;																		\
	}

// Macros for checking HRESULT return codes and logging only

#define CHECK_HR(hr, msg, severity)													\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->log(__FILE__, __LINE__, hr, msg, severity);					\
	}

#define CHECK_HR0(hr, severity, msg)												\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg);				\
	}

#define CHECK_HR1(hr, severity, msg, arg1)											\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1);			\
	}

#define CHECK_HR2(hr, severity, msg, arg1, arg2)									\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2);	\
	}

#define CHECK_HR3(hr, severity, msg, arg1, arg2, arg3)								\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3); \
	}

#define CHECK_HR4(hr, severity, msg, arg1, arg2, arg3, arg4)						\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4); \
	}

#define CHECK_HR5(hr, severity, msg, arg1, arg2, arg3, arg4, arg5)					\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5); \
	}

#define CHECK_HR6(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)			\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
	}

#define CHECK_HR7(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)			\
	if (FAILED(hr)) {																\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
	}

#define VERIFY_HR_ASSERT(hr, msg, severity)											\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->log(__FILE__, __LINE__, hr, msg, severity);			\
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT1(hr, severity, msg, arg1)									\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1);	\
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT2(hr, severity, msg, arg1, arg2)							\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT3(hr, severity, msg, arg1, arg2, arg3)						\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT4(hr, severity, msg, arg1, arg2, arg3, arg4)				\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT5(hr, severity, msg, arg1, arg2, arg3, arg4, arg5)			\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT6(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)	\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

#define VERIFY_HR_ASSERT7(hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		HRESULT hr2 = E_FAIL;														\
		if (pLog) hr2 = pLog->vlog(__FILE__, __LINE__, hr, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		_ASSERTE(!msg);																\
		return hr2;																	\
	}										

// Macro for checking a boolean condition and returning E_FAIL if it evaluates to false

#define VERIFY_COND(cond, msg, severity)											\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->log(__FILE__, __LINE__, E_FAIL, msg, severity);			\
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND0(cond, severity, msg)											\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg);			\
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND1(cond, severity, msg, arg1)										\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1);		\
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND2(cond, severity, msg, arg1, arg2)								\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2); \
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND3(cond, severity, msg, arg1, arg2, arg3)							\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3); \
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND4(cond, severity, msg, arg1, arg2, arg3, arg4)					\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4); \
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND5(cond, severity, msg, arg1, arg2, arg3, arg4, arg5)				\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND6(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)		\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
		else return E_FAIL;															\
	}

#define VERIFY_COND7(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		else return E_FAIL;															\
	}

// Macro for checking a boolean condition and returning a specific return value
// if it evaluates to false

#define VERIFY_COND_(ret, cond, msg, severity)										\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, E_FAIL, msg, severity);					\
		}																			\
		return ret;																	\
	}

#define VERIFY_COND0_(ret, cond, severity, msg)										\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg);					\
		}																			\
		return ret;																	\
	}

#define VERIFY_COND1_(ret, cond, severity, msg, arg1)								\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1);			\
		}																			\
		return ret;																	\
	}

#define VERIFY_COND2_(ret, cond, severity, msg, arg1, arg2)							\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2);		\
		}																			\
		return ret;																	\
	}

#define VERIFY_COND3_(ret, cond, severity, msg, arg1, arg2, arg3)					\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3); \
		}																			\
		return ret;																	\
	}

#define VERIFY_COND4_(ret, cond, severity, msg, arg1, arg2, arg3, arg4)				\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4); \
		}																			\
		return ret;																	\
	}

#define VERIFY_COND5_(ret, cond, severity, msg, arg1, arg2, arg3, arg4, arg5)		\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
		return ret;																	\
	}

#define VERIFY_COND6_(ret, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)	\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
		return ret;																	\
	}

#define VERIFY_COND7_(ret, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		return ret;																	\
	}

// Macro for checking a boolean condition and returning nothing if it evaluates to false

#define VERIFY_COND_V(cond, msg, severity)											\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, E_FAIL, msg, severity);					\
		}																			\
		return;																		\
	}

#define VERIFY_COND0_V(cond, severity, msg)											\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg);					\
		}																			\
		return;																		\
	}

#define VERIFY_COND1_V(cond, severity, msg, arg1)									\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1);			\
		}																			\
		return;																		\
	}

#define VERIFY_COND2_V(cond, severity, msg, arg1, arg2)								\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2);		\
		}																			\
		return;																		\
	}

#define VERIFY_COND3_V(cond, severity, msg, arg1, arg2, arg3)						\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3); \
		}																			\
		return;																		\
	}

#define VERIFY_COND4_V(cond, severity, msg, arg1, arg2, arg3, arg4)					\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4); \
		}																			\
		return;																		\
	}

#define VERIFY_COND5_V(cond, severity, msg, arg1, arg2, arg3, arg4, arg5)			\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
		return;																		\
	}

#define VERIFY_COND6_V(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)		\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
		return;																		\
	}

#define VERIFY_COND7_V(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		return;																		\
	}

// Macro for checking a boolean condition and logging an error if it evaluates to false

#define CHECK_COND(cond, msg, severity)												\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, E_FAIL, msg, severity);					\
		}																			\
	}

#define CHECK_COND0(cond, severity, msg)											\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg);					\
		}																			\
	}

#define CHECK_COND1(cond, severity, msg, arg1)										\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1);			\
		}																			\
	}

#define CHECK_COND2(cond, severity, msg, arg1, arg2)								\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2);		\
		}																			\
	}

#define CHECK_COND3(cond, severity, msg, arg1, arg2, arg3)							\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3); \
		}																			\
	}

#define CHECK_COND4(cond, severity, msg, arg1, arg2, arg3, arg4)					\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4); \
		}																			\
	}

#define CHECK_COND5(cond, severity, msg, arg1, arg2, arg3, arg4, arg5)				\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
	}

#define CHECK_COND6(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)		\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
	}

#define CHECK_COND7(cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (!(cond))																	\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			pLog->log(__FILE__, __LINE__, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, E_FAIL, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
	}

// Macros for checking an HRESULT and a boolean condition and returning HRESULT or
// E_FAIL

#define VERIFY_HR_COND(hr, cond, msg, severity)										\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->log(__FILE__, __LINE__, hr2, msg, severity);				\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND0(hr, cond, severity, msg)									\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg);				\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND1(hr, cond, severity, msg, arg1)								\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1);		\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND2(hr, cond, severity, msg, arg1, arg2)						\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2);	\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND3(hr, cond, severity, msg, arg1, arg2, arg3)					\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3);	\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND4(hr, cond, severity, msg, arg1, arg2, arg3, arg4)			\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4);	\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND5(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5)		\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5);	\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND6(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6);	\
		}																			\
		else return E_FAIL;															\
	}										

#define VERIFY_HR_COND7(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			return pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		else return E_FAIL;															\
	}										

// Macros for checking an HRESULT and a boolean condition and returning a specific
// error code if either fails

#define VERIFY_HR_COND_(ret, hr, cond, msg, severity)								\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, hr2, msg, severity);						\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND0_(ret, hr, cond, severity, msg)								\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg);						\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND1_(ret, hr, cond, severity, msg, arg1)						\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1);				\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND2_(ret, hr, cond, severity, msg, arg1, arg2)					\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2);			\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND3_(ret, hr, cond, severity, msg, arg1, arg2, arg3)			\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3);	\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND4_(ret, hr, cond, severity, msg, arg1, arg2, arg3, arg4)		\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4);	\
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND5_(ret, hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5) \
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND6_(ret, hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6) \
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
		return ret;																	\
	}										

#define VERIFY_HR_COND7_(ret, hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7) \
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		return ret;																	\
	}										

// Macros for checking an HRESULT and a boolean condition and returning nothing
// if either fails

#define VERIFY_HR_COND_V(hr, cond, msg, severity)									\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, hr2, msg, severity);						\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND0_V(hr, cond, severity, msg)									\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg);						\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND1_V(hr, cond, severity, msg, arg1)							\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1);				\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND2_V(hr, cond, severity, msg, arg1, arg2)						\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2);			\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND3_V(hr, cond, severity, msg, arg1, arg2, arg3)				\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3);	\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND4_V(hr, cond, severity, msg, arg1, arg2, arg3, arg4)			\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4);	\
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND5_V(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND6_V(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
		return;																		\
	}										

#define VERIFY_HR_COND7_V(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
		return;																		\
	}										

// Macros for checking an HRESULT and a boolean condition and logging an error
// if either fails

#define CHECK_HR_COND(hr, cond, msg, severity)										\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->log(__FILE__, __LINE__, hr2, msg, severity);						\
		}																			\
	}										

#define CHECK_HR_COND0(hr, cond, severity, msg)										\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg);						\
		}																			\
	}										

#define CHECK_HR_COND1(hr, cond, severity, msg, arg1)								\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1);				\
		}																			\
	}										

#define CHECK_HR_COND2(hr, cond, severity, msg, arg1, arg2)							\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2);			\
		}																			\
	}										

#define CHECK_HR_COND3(hr, cond, severity, msg, arg1, arg2, arg3)					\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3);	\
		}																			\
	}										

#define CHECK_HR_COND4(hr, cond, severity, msg, arg1, arg2, arg3, arg4)				\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4);	\
		}																			\
	}										

#define CHECK_HR_COND5(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5)		\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5); \
		}																			\
	}										

#define CHECK_HR_COND6(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6); \
		}																			\
	}										

#define CHECK_HR_COND7(hr, cond, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7)	\
	if (FAILED(hr) || !(cond))														\
	{																				\
		CILXLogHelper3* pLog = CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);	\
		if (pLog) {																	\
			HRESULT hr2 = FAILED(hr) ? hr : E_FAIL;									\
			if (!(cond))															\
				pLog->log(__FILE__, __LINE__, hr2, _T("Condition failed: \"") _T(#cond) _T("\""), severity); \
			pLog->vlog(__FILE__, __LINE__, hr2, severity, msg, arg1, arg2, arg3, arg4, arg5, arg6, arg7); \
		}																			\
	}										

// Message severities - Duplicates severities defined in IlxCommon.idl
// and IlxLog.idl.

enum ILXLOGHELPER_SEVERITY
{
	ILXLOG_SUCCESS						= 0,
	ILXLOG_INFO							= 1,
	ILXLOG_WARNING						= 2,
	ILXLOG_ERROR						= 3
};

// Forward declarations
interface IILXLog;

// Forward delcare the IXMLDOMNode interface
namespace MSXML
{
	struct __declspec(uuid("2933bf80-7b36-11d2-b20e-00c04f983e60"))
	/* dual interface */ IXMLDOMNode;
};

//*****************************
// CILXLogHelper2
//*****************************

class DLLPort CILXLogHelper2
	//
	// Before you call any other methods make sure that logging
	// has been initialized.
	//
	// An EXE should initialize logging by doing the following:
	//
	// Step 1:	Call initialize()
	// Step 2:	Call either loadConfig() or useLogFile()
	//
	// A DLL (plain or ActiveX control) should only do these
	// steps if they haven't been done already.  Here is an
	// example:
	//
	// if (!GetLog().isInitialized())
	// {
	//		hr = GetLog().initialize(_T("ILXQuote"));
	//		_ASSERTE(SUCCEEDED(hr));
	//		hr = GetLog().useLogFile(_T("ILXQuote.log"));
	//		_ASSERTE(SUCCEEDED(hr));
	// }
	//
	// If you call initialize(), you MUST call shutdown() before the
	// application closes or the DLL module is unloaded.
	//
{
public:
	//
	// Initialization Methods
	//

	HRESULT initialize(	const TCHAR* szSubsystem, 
						HMODULE hResourceModule = NULL);
		//
		// Must be called before any other method.
		//
		// PARAMETERS:
		//
		//	szSubsystem - The logical module or subsystem name.
		//		The name is displayed in the log file and in
		//		error dialogs.  It helps to identify the module
		//		where the error occurred.
		//

	HRESULT loadConfig(MSXML::IXMLDOMNode* piLogConfigNode);
		//
		// Loads the log settings from the ILXLOG XML node.
		// This should be called after initialize() but before
		// log() or msgBox().  See the "IlxLog - XML Format.txt"
		// file for a description.
		//
		// You cannot call both loadConfig() and useLogFile().
		//

	HRESULT useLogFile(	const TCHAR* szLogFilename, 
						long nMaxFileSize = 0);
		//
		// Tells the logging system to use the default log
		// settings and a log file.  The default settings displays
		// a message box for errors and writes all errors and
		// warnings to the debug output and a log file.
		//
		// You cannot call both loadConfig() and useLogFile().
		//
		// PARAMETERS:
		//
		//	szLogFilename - The filename of the log file to
		//		create.  The log file will be created in the
		//		RDC Log Directory specified by the
		//		HKCU\Software\ILX\RDC\Log Directory registry
		//		key.
		//
		//		If the log file already exists, the file is
		//		backed up by appending ".bak" to the filename
		//		and a new file gets created.
		//
		//	nMaxFileSize - The maximum size that the log file
		//		can become.  If the log file exceeds this size,
		//		it is copied to a backup file (".bak" is appended
		//		to the name) and then a new log file gets created.
		//		If 0, there is no max size (log file is infinite).
		//		If -1, the default max size is used (65536 bytes).
		//

	bool isInitialized() const;
		//
		// Returns true if initialize() and either loadConfig() or
		// useLogFile() have been called.  A DLL can check this to
		// see if it needs to initialize logging.
		//

	void shutdown();
		//
		// Must be called by the same thread that calls initialize()
		// before the module exits (EXE exits or DLL is unloaded).
		//

	HRESULT addResourceModule(HMODULE hModule);
	HRESULT addResourceFile(const TCHAR* szFilename);
		//
		// Adds an executable (EXE or DLL) which has message compiled
		// resources.  This allows logging to find the messages
		// associated with HRESULTs when log() and msgBox() are called.
		// 

	//
	// Logging Methods
	//

	void	log(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ const TCHAR* szMessage, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity);

	HRESULT log(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ HRESULT hr, 
					/*[in]*/ const TCHAR* szMessage, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity);

	HRESULT log(	/*[in]*/ const char* szSubsystem, 
					/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ HRESULT hr, 
					/*[in]*/ const TCHAR* szMessage, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity);

	void	vlog(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity,
					/*[in]*/ const TCHAR* szFormat, 
					/*[in]*/ ...);

	HRESULT vlog(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ HRESULT hr, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity,
					/*[in]*/ const TCHAR* szFormat,
					/*[in]*/ ...);

	HRESULT vlog(	/*[in]*/ const char* szSubsystem, 
					/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ HRESULT hr, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity,
					/*[in]*/ const TCHAR* szFormat,
					/*[in]*/ ...);
		//
		// Logs a message.
		//
		// RETURNS: The hr parameter if one is specified.
		//
		// PARAMETERS:
		//
		//  szSubsystem - The logical subsystem where the message
		//		occurred.  You should not need to specify this
		//		unless you need to override the subsystem used by
		//		initialize().
		//
		//	szPath - Path to the source file where this function
		//		is being called.  This should be __FILE__.
		//
		//	nLine - Path to the source file line where this function
		//		is being called.  This should be __FILE__.
		//
		//	hr - Error code of the error message to log.  The
		//		error code will be translated to its corresponding
		//		error string.
		//
		//	szMessage - Message to be logged.
		//
		//	nSeverity - One of: ILXLOG_SUCCESS, ILXLOG_INFO,
		//		ILXLOG_WARNING, or ILXLOG_ERROR
		//

	void msgBox(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ HRESULT hr);

	void msgBox(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ const TCHAR* szMessage, 
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity);

	void vmsgBox(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity,
					/*[in]*/ const TCHAR* szFormat, 
					/*[in]*/ ...);
		//
		// Displays an error dialog box.
		//
		// RETURNS: The hr parameter if one is specified.
		//
		// PARAMETERS:
		//
		//	szPath - Path to the source file where this function
		//		is being called.  This should be __FILE__.
		//
		//	nLine - Path to the source file line where this function
		//		is being called.  This should be __FILE__.
		//
		//	hr - Error code of the error message to log.  The
		//		error code will be translated to its corresponding
		//		error string.
		//
		//	szMessage - Message to be logged.
		//
		//	nSeverity - One of: ILXLOG_SUCCESS, ILXLOG_INFO,
		//		ILXLOG_WARNING, or ILXLOG_ERROR
		//

	HRESULT getILXLogUnk(IUnknown** ppiIlxLogUnk);
		//
		// Returns the internal IILXLog COM object.  This is
		// dangerous because the object aggregates the free-threaded
		// marshaller which means that no marshalling will occur
		// when you call methods on the object.  If another thread
		// tries to log messages at the same time then corruption
		// will result.
		//

	~CILXLogHelper2();
		//
		// Only used internally.
		//

	static CILXLogHelper2& GetLogSafe(long nIlxLogApiVersion = ILX_LOG_API_VERSION);
		//
		// Returns the global log helper singleton object. If the
		// version passed in does not match the one compiled into
		// IlxLog.dll then the API is not compatible and a message
		// box will be displayed.  However, the singleton is still
		// returned and undefined behavior can result.  This is
		// here for backward compatibility.
		//

	static CILXLogHelper2* GetLogSafePtr(long nIlxLogApiVersion = ILX_LOG_API_VERSION);
		//
		// Same as GetLogSafe() except a NULL pointer is returned
		// if the versions don't match.  This is the safest mechanism.
		//

protected: // Cannot be created externally. You must call GetLog().
	CILXLogHelper2();

private: // NOT IMPLEMENTED
	CILXLogHelper2& operator=(const CILXLogHelper2&);
	CILXLogHelper2(const CILXLogHelper2&);

protected: // DATA
	BSTR				m_bstrSubsystem;
		//
		// The logical name of the subsystem to be prefixed to all logged
		// messages.
		//

	IILXLog*			m_piIlxLog;
		//
		// The IILXLog object used to implement the actual logging.
		//

	CRITICAL_SECTION	m_csLogging;
		//
		// For thread safety.
		//

	bool				m_bIsInitialized;
		//
		// TRUE if initialize() and either useLogFile() or loadConfig()
		// have been called.
		//
};

class DLLPort CILXLogHelper3 : public CILXLogHelper2
{
public:
	void vlogCrash(	/*[in]*/ const char* szPath, 
					/*[in]*/ long nLine,
					/*[in]*/ EXCEPTION_POINTERS* pExceptionInfo,
					/*[in]*/ const TCHAR* szSearchPath,
					/*[in]*/ ILXLOGHELPER_SEVERITY eSeverity,
					/*[in]*/ const TCHAR* szFormat,
					/*[in]*/ ...);

	static CILXLogHelper3& GetLogSafe(long nIlxLogApiVersion = ILX_LOG_API_VERSION);
		//
		// Returns the global log helper singleton object. If the
		// version passed in does not match the one compiled into
		// IlxLog.dll then the API is not compatible and a message
		// box will be displayed.  However, the singleton is still
		// returned and undefined behavior can result.  This is
		// here for backward compatibility.
		//

	static CILXLogHelper3* GetLogSafePtr(long nIlxLogApiVersion = ILX_LOG_API_VERSION);
		//
		// Same as GetLogSafe() except a NULL pointer is returned
		// if the versions don't match.  This is the safest mechanism.
		//

private: // Cannot be created externally. You must call GetLog().
	CILXLogHelper3();

private: // NOT IMPLEMENTED
	CILXLogHelper3& operator=(const CILXLogHelper3&);
	CILXLogHelper3(const CILXLogHelper3&);

private: // DATA

	friend CILXLogHelper2;

	static CILXLogHelper3 s_singleton;
		//
		// The singleton object returned by GetLog().  One of these gets
		// created once per module (DLL or EXE).
		//
};

//
// Returns the global log helper singleton object. Example:
//
// GetLog().initialize((_T("ActiveILX"));
//
inline CILXLogHelper3& GetLog()
{
	return CILXLogHelper3::GetLogSafe(ILX_LOG_API_VERSION);
}

inline CILXLogHelper3* GetLogSafe()
{
	return CILXLogHelper3::GetLogSafePtr(ILX_LOG_API_VERSION);
}

#endif  // INCLUDED_ILX_LOG_HELPER2_H