///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2000 ILX Systems.  All Rights Reserved.
//
//      $Header: /u/4/m/me133/cvs/Common/XmlElement.h,v 1.2 2004/09/07 06:08:58 me133 Exp $
//
//    File name:	XmlElement.h
//   Created By:	Marc Eaddy
//   Created on:	03/07/2000 11:38:25 AM
//
//      Purpose:	Provides XML persistence for an object and its children
//
//  Description:	Converts an XML object model into a user-defined
//					class hierarchy.  For example, the following XML
//
//					<company NAME="ILX">
//						<department NAME="Accounting">
//							<manager NAME="Juniper"/>
//							<supervisor NAME="Jeff"/>
//							<employees>
//								<employee NAME="Jack"/>
//								<employee NAME="Jill"/>
//							</employees>
//						</department>
//					</company>
//
//					would generate a generic XML object model at runtime.
//					However, it is usually more convenient to deal with
//					higher level abstractions such as the CCompany,
//					CDepartment, CManager, CSupervisor, CEmployees,
//					and CEmployee classes.  That would allow us to provide
//					methods like CManager.reviewPerformance(CEmployee)
//					which are not available in XML.
//
//					CXmlElement provides a mechanism for mapping an XML
//					object model into a user-defined class hierarchy
//					(and back).  A class that derives from CXmlElement
//					provides a list of class prototypes that can be
//					children.  For example, the child prototypes of
//					CDepartment would be CManager, CSupervisor, and
//					CEmployees.  When an XML element is encountered
//					a new instance is created using the prototype.
//					For example, if the <manager> element is found,
//					the CManager prototype will be used to create
//					a new CManager object.  The new object will then
//					be told to read in its attributes and children.
//					The new object is then added to the list of children
//					for the CDepartment object.
//
//					Sometimes you want one XML element to map to more
//					than one class.  For example, a <DEST> element
//					represents a generic log destination (CLogDestination).
//					By specifying the TYPE attribute, we can map the
//					DEST element to a subclass of CLogDestination.
//					For example,
//
//					<DEST TYPE="MsgBox"/>	- Maps to CMsgBoxDestination
//					<DEST TYPE="LogFile"/>	- Maps to CLogFileDestination
//
//					We could have done the same thing by introducing
//					new element such as <MSG_BOX_DEST> or <LOG_FILE_DEST>
//					but I think the above is cleaner.
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: XmlElement.h,v $
 * Revision 1.2  2004/09/07 06:08:58  me133
 * o Second checkin to CVS
 * 
 * 
 * 41    11/02/01 6:30p Marce
 * o Now if you specify 0 or "" for setSourceFile() the attribute will be
 * deleted
 * 
 * 40    11/02/01 5:17p Zakirh
 * added removeSourceFile
 * 
 * 39    11/02/01 4:33p Marce
 * o Made some methods public
 * 
 * 38    11/02/01 1:02p Marce
 * o Moved constants back into source file because they are causing
 * ILXCommon to not compile
 * o Added getSourceFile() and setSourceFile()
 * 
 * 34    10/30/01 12:06p Marce
 * o Fixed compile bug
 * o Changed method signature to avoid extra memory allocation and
 * copying, and to allow you to specify any variant as a value not just a
 * string
 * 
 * 33    10/30/01 11:39a Zakirh
 * added setAttribute
 * 
 * 32    10/25/01 9:49p Marce
 * o Fixed compile error
 * 
 * 31    10/25/01 9:34p Marce
 * o Now overwrites the SRC file when saving
 * o Now creates a new XML document if needed when saving
 * 
 * 30    10/23/01 6:20p Marce
 * o Changed name from loadFromSource() to loadChildren() for clarity
 * 
 * 29    10/23/01 6:04p Marce
 * o Added support for specifiying a separate file to hold child elements
 * using the SRC attribute
 * 
 * 28    10/22/01 2:35p Marce
 * o Broke out functionality so it can be overridden.  Added loadChild(),
 * afterLoad(), and beforeSave().
 * o Made some data members protected
 * 
 * 27    10/22/01 12:35p Marce
 * o One version of load() doesn't have to be virtual since it calls the
 * other version
 * o Added afterLoad() virtual method
 * 
 * 26    10/19/01 7:18p Marce
 * o Now allows you to load a file or a node
 * 
 * 25    10/19/01 4:42p Marce
 * o Added getElementName()
 * 
 * 24    9/24/01 12:25p Marce
 * o Changed to isMyChild() for clarity
 * 
 * 23    9/24/01 12:22p Marce
 * o Added isChild()
 * 
 * 22    8/17/01 3:17p Marce
 * o Now saved XML looks much better
 * 
 * 21    8/16/01 7:15p Marce
 * o Added ability to get any attribute
 * 
 * 19    7/19/01 2:33p Marce
 * o Added ability to delete and reparent children
 * o insertChild() defaults to append the child
 * 
 * 18    7/19/01 1:53p Marce
 * o Added delete and reparent
 * 
 * 17    7/17/01 6:56p Marce
 * o Now compiles in isolation
 * 
 * 16    7/16/01 5:28p Marce
 * o clear() should clear the attributes as well
 * 
 * 15    7/16/01 5:27p Marce
 * o Added clear() method
 * 
 * 14    7/05/01 3:19p Marce
 * o Now insertChild() is virtual
 * 
 * 13    7/03/01 7:57p Marce
 * o Now a child can get its parent
 * o Fixed order of member initialization in constructor
 * 
 * 12    7/03/01 6:24p Marce
 * o Now implements save()
 * o Now allows you to specify a factory creation function instead of
 * using prototypes.  This is necessary if the class's constructor calls
 * addPrototype() with the same name as the class which results in an
 * infinite recursion.
 * 
 * 11    12/01/00 6:46p Marce
 * o Added ctor
 * 
 * 10    11/10/00 1:12p Marce
 * o Added comments
 * 
 * 8     10/04/00 5:36p Marce
 * o Provided a check to prevent multiple load() calls which are usually
 * not expected
 * 
 * 7     10/03/00 8:07p Marce
 * o Now provides access to subtype
 * 
 * 6     10/03/00 6:36p Marce
 * o Added support for using the NAME attribute to specify subclasses
 * 
 * 5     3/08/00 1:35p Marce
 * o Now all class uses a common base class that implements XML
 * persistence
 */

#ifndef INCLUDED_XML_ELEMENT_H
#define INCLUDED_XML_ELEMENT_H

#include "CaseInsensitiveString.h"	// tstring

#include <comdef.h>					// _variant_t
#include <vector>					// STL vector<>
#include <map>						// STL map<>

// Forward declarations
class CXmlElement;

// Forward delcare the IXMLDOMNode class
namespace MSXML
{
	struct __declspec(uuid("2933bf80-7b36-11d2-b20e-00c04f983e60"))
	/* dual interface */ IXMLDOMNode;
	struct __declspec(uuid("2933bf81-7b36-11d2-b20e-00c04f983e60"))
	/* dual interface */ IXMLDOMDocument;
};

// Type defines
typedef std::vector<CXmlElement*>							TypeVecXmlElementPtrs;
typedef std::vector<const CXmlElement*>						TypeVecXmlElementCPtrs;
typedef std::map<tstring, _variant_t, TypeFuncLessTString >	TypeMapStr2Var;

typedef CXmlElement* (*PFN_FACTORY_METHOD)(const CXmlElement& elemThis,
										const TCHAR* szXmlElementName,
										const CComVariant& varAttributeValue);

//*****************************
// CXmlElement
//*****************************

class CXmlElement
{
public:

	CXmlElement(const TCHAR* szXmlTagName, 
				const TCHAR* szXmlTagName2 = _T(""),
				const TCHAR* szSubType = _T(""),
				bool bAllowMultipleLoads = false,
				bool bValidateXmlTagOnLoad = true);

	CXmlElement(const TCHAR* szXmlTagName, 
				bool bAllowMultipleLoads);
		//
		// Creates an empty CXmlElement object.
		//
		// PARAMTERS:
		//
		//	szXmlTagName - The XML tag name that this CXmlElement
		//		corresponds to.  For example, a class called
		//		CEventTable that derives from CXmlElement might
		//		specify the tag name as "EVENT_TABLE" in its
		//		constructor initialization list:
		//
		//		CEventTable::CEventTable()
		//			: CXmlElement(_T("EVENT_TABLE"))
		//		{ }
		//
		//	szXmlTagName2 - An optional second XML tag name that
		//		this CXmlElement corresponds to.  This is useful
		//		when you want to have two separate XML elements
		//		that refer to the same high-level class.  For
		//		example, EVENT_TABLE2.  The default is the empty
		//		string.
		//
		//	szSubType - The XML "TYPE" attribute that this element
		//		corresponds to.  See the example in the header
		//		description.  The default is the empty string.
		//
		//	bAllowMultipleLoads - Whether or not load() can be
		//		called more than once.  The default is false.
		//		You might want to support this if you want to
		//		initialize the object from multiple XML sources.
		//		For example, one XML source might specify the
		//		defaults and another might override the defaults.
		//

	CXmlElement(const CXmlElement&);			// COPY CTOR
	CXmlElement& operator=(const CXmlElement&);	// ASSIGNMENT
	virtual ~CXmlElement();						// DTOR

public:

	//----------------------
	// Persistence methods
	//----------------------

	HRESULT load(VARIANT varFileNameOrNode);
	virtual HRESULT load(MSXML::IXMLDOMNode* piNode);
		//
		// Loads the object's attributes and children from the XML
		// node.
		//
		// load(IXMLDOMNode*) does the following:
		//		1. Loads the attributes
		//
		//		For each child node:
		//			2. Calls loadChild()
		//			3. Calls insertChild()
		//
		//		4. Calls afterLoad()
		//
		// PARAMETERS:
		//
		//	piNode - The XML nodes element name must match the
		//		szXmlTagName specified in the constructor.  For
		//		example, if the element tag name is <EVENT_TABLE>
		//		then the only subclass that can load from the
		//		node is CEventTable.
		//

	virtual HRESULT loadChildren(const TCHAR* szFilePath);
		//
		// Subclasses can override this to takeover loading children
		// (or other things) when the SRC attribute is encountered.
		// Default is to load children from the file specified.
		//

	virtual HRESULT loadChildren(MSXML::IXMLDOMNode* piParentNode);
		//
		// Subclasses can override this to takeover how children are
		// loaded.  Default is to call loadChild() for each child
		// element.
		//

	HRESULT save(VARIANT varFileNameOrNode, long nIndentLevel = 0);
	virtual HRESULT save(MSXML::IXMLDOMNode* piNode, long nIndentLevel = 0);
		//
		// Saves the layout element to the XML node.
		//
		// PARAMETERS:
		//
		//	piNode - 
		//

	virtual HRESULT saveChildren(const TCHAR* szFilePath, long nIndentLevel = 0);
		//
		// Subclasses can override this to takeover saving children
		// (or other things) when the SRC attribute is encountered.
		// Default is to save children from the file specified.
		//

	virtual HRESULT saveChildren(MSXML::IXMLDOMNode* piParentNode, long nIndentLevel = 0);
		//
		// Subclasses can override this to takeover how children are
		// saved.
		//

	virtual void clear();
		//
		// Resets the state of this element.  Clears all children,
		// resets the loaded flag to false, and clears all the
		// attributes.
		//

	//------------------
	// Child operations
	//------------------

	virtual CXmlElement* insertChild(CXmlElement* pChild, int nIndex = -1);
		// 
		// Insert a layout element child into this element.
		//
		// RETURNS: Newly added child.
		//
		// PARAMETERS:
		//
		//	pChild - The child element to be inserted.  The element
		//		ptr memory will be owned by this node.
		//	nIndex - Position in the child collection to insert the
		//		child.  Must be either -1 or between 0 and getNumChildren().
		//		If -1, then the child will be appended to the end of 
		//		the collection.
		//

	void removeChildren();
		// Removes all the children

	void deleteChild(int nIndex);
	void deleteChild(CXmlElement* pChild);
		//
		// Deletes the child
		// 

	void reparentChild(int nIndex, CXmlElement* pNewParent, int nNewIndex = -1);
	void reparentChild(CXmlElement* pChild, CXmlElement* pNewParent, int nNewIndex = -1);
		//
		// Removes the child and adds it to the new parent.
		//
		// Calls pNewParent->insertChild(pChild, nNewIndex)
		//
		// PARAMETERS:
		//
		//	nIndex - The index of the child to reparent.
		//	pChild - The ptr to the child to reparent.
		//	pNewParent - The new parent for the child.
		//	nNewIndex - The new index for the child.
		//

	const TypeVecXmlElementPtrs& getChildren() const;
	TypeVecXmlElementPtrs& getChildren();
		// Retrieves all the children of this element.

	const CXmlElement* getChild(int nIndex) const;
	CXmlElement* getChild(int nIndex);
		//
		// Retrieves a specific child of this element.
		//
		// PARAMETERS:
		//
		//	nIndex - Must be between 0 and getNumChildren().
		//

	long getNumChildren() const;
		// Retrieves the number of children for this element.

	const CXmlElement* getParent() const;
	CXmlElement* getParent();

	bool isMyChild(CXmlElement* pElement) const;
		//
		// Returns true if pElement is a child of this element.
		//

	//------------------
	// Identity operations
	//------------------

	const tstring& getElementName() const;
		// Retrieves the element tag name of this element

	const tstring& getSubType() const;
		// Retrieves the subtype of this element

	const OLECHAR* getNameAttr() const;
		// Retrieves the NAME attribute

	bool isLoaded() const;
		// TRUE if load() has been called, FALSE otherwise

	void setSourceFile(const TCHAR* szFilePath);
	const TCHAR* getSourceFile() const;

	//------------------
	// Attributes
	//------------------

	const _variant_t& getAttribute(const tstring& strName) const;
	_variant_t& getAttribute(const tstring& strName);

	void setAttribute(const TCHAR* szName, const _variant_t& varValue);

protected: // SUBCLASS HELPER FUNCTIONS

	virtual HRESULT loadChild(MSXML::IXMLDOMNode* piChildNode);
		//
		// Subclasses can override this to takeover loading children
		// nodes and creating children objects.
		//

	virtual HRESULT afterLoadAttributes();
		//
		// Subclasses can override this to initialize data members
		// with values from the attribute map.
		//

	virtual HRESULT afterLoad();
		//
		// Subclasses can override this to initialize data members
		// with values from the attribute map.
		//

	virtual HRESULT beforeSave();
		//
		// Subclasses can override this to initialize the attribute
		// map with values from data members.
		//

	void setFactoryMethod(PFN_FACTORY_METHOD pfnFactoryMethod);
		//
		// Sets the factory method used to create new child
		// elements.  If the factory method isn't specified then
		// the child prototypes will be used instead.
		// 

	void addChildPrototype(const CXmlElement* element);
		//
		// Adds a child prototype to the prototype list
		// for this element which will be used during load() to 
		// create the appropriate layout element subclass
		// based on the XML element name encountered.
		// 
		// The subclass must call this method in its constructor
		// for each child element that it can have.  Note: Subclasses
		// MUST NOT call this method in their copy constructor.
		//

	virtual CXmlElement* clone() const;
		//
		// Clones this element.
		//
		// This implements the Factory Method design pattern
		// by implementing a "virtual" constructor, which
		// each subclass must override to return a copy of
		// itself.  Thus, we can make a copy of a persisted
		// element without having to know what type the
		// persisted element is.
		//

private:	// HELPER FUNCTIONS

	CXmlElement* createNew(	const TCHAR* szXmlElementName,
							const CComVariant& varAttributeValue) const;
		// Implements the Abstract Factory design pattern by allowing
		// a layout element subclass to be created using its XML element
		// name.  The list of child prototypes is searched to find an
		// element that matches the name specified.  If the correct
		// prototype is found, it is cloned and the clone is returned.

	void removePrototypes();
		// Removes all the child prototypes

	void copyChildrenAndPrototypes(const CXmlElement& that);
		// Copies the children and child prototypes of the layout
		// element specified.

protected: // DATA
	tstring						m_strXmlTagName;
	CXmlElement*				m_pParent;
	TypeMapStr2Var				m_mapAttributes;
	TypeVecXmlElementPtrs		m_vecChildren;

private: // DATA

	tstring						m_strXmlTagName2;
	tstring						m_strSubType;
	TypeVecXmlElementCPtrs		m_vecChildPrototypes;
	bool						m_bAllowMultipleLoads;
	bool						m_bIsLoaded;
	bool						m_bValidateXmlTagOnLoad;
	PFN_FACTORY_METHOD			m_pfnFactoryMethod;
};

// Helper functions

HRESULT GetXmlNodeFromVariant(	VARIANT varFileNameOrXmlNode,
								MSXML::IXMLDOMNode** ppiXmlNode,
								BSTR bstrRootElementForNewDocument = 0,
								MSXML::IXMLDOMDocument** ppiXmlDocument = 0);

#endif // INCLUDED_XML_ELEMENT_H