///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Patcher/MetadataIterators.h 12    12/14/05 2:16a Eaddy $
//
//    File name:	MetadataIterators.h
//   Created By:	Marc Eaddy
//   Created on:	10/10/04
//
//      Purpose:	STL-like iterators for Unmanaged Metadata
//
//  Description:	Wraps the IMetadataImport->EnumXXX methods in
//					convenient STL iterators.
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Patcher/MetadataIterators.h $
 * 
 * 12    12/14/05 2:16a Eaddy
 * o Even if we don't support emitting new types, methods, and fields, 
 * we still need to fix them up.  And we can't assume that they can
 * be iterated in lock step since they may have been reordered.
 * Actually, this only really happens when the build description file
 * doesn't match the executable because the exe was built using
 * VS.
 * 
 * 11    12/09/05 9:33p Eaddy
 * o Added support for introducing fields but of course this doesn't work
 * either because the CLR requires DefineField to be called in
 * ModuleLoadFinished
 * 
 * 10    12/09/05 9:03p Eaddy
 * o Minor changes
 * 
 * 9     12/09/05 4:34p Eaddy
 * o methodDef_iterator now uses CMethodInfo to store method props
 * o Now all iterators use the same ctor signature to make the
 * MetadataDiff template method happy and also to ensure vital
 * params are available to subtypes
 * 
 * 8     12/09/05 1:34p Eaddy
 * o Improved log messages
 * 
 * 7     12/08/05 10:34p Eaddy
 * o Now really fixed problem with changing a method's local variables
 * 
 * 6     12/08/05 8:29p Eaddy
 * o Now compares signatures correctly
 * o Now supports changing a method's local variables
 * 
 * 5     12/07/05 11:37p Eaddy
 * o Added assemblyRef iter
 * 
 * 4     12/07/05 7:54p Eaddy
 * o Added iterators for typerefs and member refs
 * o Added emit()
 * o Now each iterator provides a way to obtain, compare, and print its
 * metadata value, and find an element using its token
 * 
 * 3     5/28/05 1:20p Eaddy
 * Changed copyright
 * Revision 1.2  2004/10/10 06:07:45  me133
 * o Added header comment
 */

#ifndef INCLUDE_METADATA_ITERATORS
#define INCLUDE_METADATA_ITERATORS

#include <cor.h>			// HCORENUM

#include <atlcomcli.h>		// CComPtr

#include <functional>		// STL less<>
#include <iterator>			// STL iterator
#include <string>			// STL string
#include <list>				// STL list

#include "ILHelper.h"		// SignaturesAreEqual()

#include "MetadataHelpers.h"	// Sig2String()

#include "MethodInfo.h"		// CMethodInfo

// MACROS

#define NumItems(s) (sizeof(s) / sizeof(s[0]))

// CONSTANTS

static const int nENUM_BUFFER_SIZE = 50;

// Type definitions

typedef std::pair<mdToken, mdToken>	TypePairTokens;
typedef std::list<TypePairTokens>	TypeListTokenPairs;

//////////////////////
// HCORENUM_iterator
//////////////////////
//
// Base class used by all the iterators
//
template<typename Def> 
	class HCORENUM_iterator : 
		public std::iterator<std::input_iterator_tag,
                                            Def>
{
public:

	HCORENUM_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent) 
		: m_spiImport(piImport)
		, m_pbLoadAddress(pbLoadAddress)
		, m_bIsImage(bIsImage)
		, m_tokParent(tokParent)
		, m_hEnum(0)
		, m_ulCount(0)
		, m_ary()
		, m_ulCurrent(0)
		, m_ulTotal(0)
    { }

	HCORENUM_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, Def def, bool) 
		: m_spiImport(piImport)
		, m_pbLoadAddress(pbLoadAddress)
		, m_bIsImage(bIsImage)
		, m_tokParent(mdTokenNil)
		, m_hEnum(0)
		, m_ulCount(0)
		, m_ary()
		, m_ulCurrent(0)
		, m_ulTotal(0)
	{ 
		m_ary[0] = def;
		m_ulTotal = 1;
		m_ulCount = 1;
		initCurrent();
	}

    HCORENUM_iterator(const HCORENUM_iterator& that)
		: m_spiImport(that.m_spiImport)
		, m_pbLoadAddress(that.m_pbLoadAddress)
		, m_bIsImage(that.m_bIsImage)
		, m_tokParent(that.operator*())
		, m_hEnum(NULL)
		, m_ulCount(0)
		, m_ary()
		, m_ulCurrent(0)
		, m_ulTotal(0)
	{ }

	~HCORENUM_iterator()
	{
		if (m_spiImport && m_hEnum)
		{
			m_spiImport->CloseEnum(m_hEnum);
			m_hEnum = NULL;
		}
	}

	// Increments the current element. If we've reached the
	// end of this batch, get the next batch.
    HCORENUM_iterator& operator++()
    {
		if (++m_ulCurrent >= m_ulCount)
		{
			getNextBatch();
		}
		else
		{
			initCurrent();
		}

        return *this;
    }

    const Def& operator*() const
    {
		_ASSERTE(m_ulCurrent < m_ulCount && m_ulCurrent < NumItems(m_ary));
        return m_ary[m_ulCurrent];
    }

    Def operator*()
    {
		_ASSERTE(m_ulCurrent < m_ulCount && m_ulCurrent < NumItems(m_ary));
        return m_ary[m_ulCurrent];
    }

	const Def getToken() const
	{
		return this->operator*();
	}

	bool find(mdToken tokToFind, const TypeListTokenPairs& listNewOrReorderedTokens)
	{
		// First see if the token has been re-mapped
		for(TypeListTokenPairs::const_iterator it = listNewOrReorderedTokens.begin();
			it != listNewOrReorderedTokens.end();
			++it)
		{
			if ((*it).first == tokToFind)
			{
				tokToFind = (*it).second;
				break;
			}
		}

		// Now find the metadata element for the token
		return find(tokToFind);
	}

	bool find(mdToken tokToFind)
	{
		for(this->reset(); !this->end(); this->operator++())
		{
			if (tokToFind == this->getToken())
			{
				return true;
			}
		}

		return false;
	}

	bool end() const
	{
		return m_ulCount == 0;
	}

	void reset()
	{
		if (m_spiImport && m_hEnum)
		{
			m_spiImport->CloseEnum(m_hEnum);
			m_hEnum = NULL;
		}

		m_ulCount = 0;
		m_ulCurrent = 0;
		m_ulTotal = 0;
		getNextBatch();
	}

	ULONG count() const
	{
		_ASSERTE(end());
		return m_ulTotal;
	}

	IMetaDataImport* getImport()
	{
		return m_spiImport.p; // Caller must AddRef()/Release()
	}

	LPCBYTE getLoadAddress()
	{
		return m_pbLoadAddress;
	}

protected:
	void getNextBatch()
	{
		// Restart the current position in the array
		m_ulCurrent = 0;

		getNextBatchImpl();

		m_ulTotal += m_ulCount;

		if (!end())
			initCurrent();
	}

	// Init current is called to allow a derived class to initialize any data
	// it may keep that's associated with the current element. When the iterator
	// is incremented, we need to make sure the data is initialized.
	virtual void initCurrent()
	{ }

	virtual void getNextBatchImpl() = 0;

private: // NOT IMPLEMENTED
	HCORENUM_iterator& operator=(const HCORENUM_iterator& that);

    bool operator==(const HCORENUM_iterator& that) const
    {
		return	m_hEnum == that.m_hEnum &&
				m_ulCurrent == that.m_ulCurrent &&
				m_ulCount == that.m_ulCount &&
				m_ulTotal == that.m_ulTotal;
    }

    bool operator!=(const HCORENUM_iterator& that) const
    {
        return !(*this == that);
    }

protected: // DATA
	CComPtr<IMetaDataImport> m_spiImport;
	LPCBYTE m_pbLoadAddress;
	bool m_bIsImage;
	mdToken m_tokParent;
    HCORENUM m_hEnum;
    Def m_ary[nENUM_BUFFER_SIZE];
	ULONG m_ulCount;

private:	// DATA
	ULONG m_ulCurrent;
	ULONG m_ulTotal;
};

/////////////
// SAssemblyProp
/////////////

struct SAssemblyProps
{
	WCHAR name[2048]; // 2048 from alink.h
	const void* pbPubKeyOrTok;
	ULONG nPubKeyLen;
	ASSEMBLYMETADATA assemblyMetadata;
	const void* pbHashBlob;
	ULONG nHashLen;
	DWORD dwFlags;

public:

	SAssemblyProps::SAssemblyProps()
		: name()
		, pbPubKeyOrTok(0)
		, nPubKeyLen(0)
		, assemblyMetadata()
		, pbHashBlob(0)
		, nHashLen(0)
		, dwFlags(0)
	{ }

	const WCHAR* toString() const
	{
		return name;
	}

	const WCHAR* toDump() const
	{
		return 0;
	}

	friend bool operator==(const SAssemblyProps& lhs, const SAssemblyProps& rhs)
	{
		// TODO: What about other assembly props?
		return wcscmp(lhs.name, rhs.name) == 0;
	}

	friend bool operator<(const SAssemblyProps& lhs, const SAssemblyProps& rhs)
	{
		// TODO: What about other assembly props?
		return wcscmp(lhs.name, rhs.name) < 0;
	}
};

/////////////////////
// assemblyRef_iterator
/////////////////////

class assemblyRef_iterator : public HCORENUM_iterator<mdAssemblyRef>
{
public:
	typedef HCORENUM_iterator<mdAssemblyRef> _Mybase;

	assemblyRef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);

	const WCHAR* getName() const;
	const SAssemblyProps& getValue() const;

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

	friend bool operator==(const assemblyRef_iterator& lhs, const assemblyRef_iterator& rhs)
	{
		return lhs.m_assemblyProps == rhs.m_assemblyProps;
	}

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();

private: // NOT IMPLEMENTED
	assemblyRef_iterator(const assemblyRef_iterator& it); 
	assemblyRef_iterator& operator=(const assemblyRef_iterator& that);

private: // DATA
	CComPtr<IMetaDataAssemblyImport> m_spiAssemblyImport;
	SAssemblyProps m_assemblyProps;
};

/////////////
// SStrProp
/////////////

struct SStrProp
{
	std::wstring str;

public:

	SStrProp::SStrProp(const WCHAR* sz)
		: str(sz)
	{ }

	SStrProp::SStrProp()
		: str()
	{ }

	const WCHAR* toString() const
	{
		return str.c_str();
	}

	const WCHAR* toDump() const
	{
		return 0;
	}

	friend bool operator<(const SStrProp& lhs, const SStrProp& rhs)
	{
		return lhs.str < rhs.str;
	}
};

/////////////////////
// typeDef_iterator
/////////////////////

class typeDef_iterator : public HCORENUM_iterator<mdTypeDef>
{
public:
	typedef HCORENUM_iterator<mdTypeDef> _Mybase;

	typeDef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);

	bool isInterface() const;
	bool isDelegate() const;
	const WCHAR* getName() const;
	const SStrProp getValue() const;

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

	static HRESULT getProps(IMetaDataImport* piImport, mdTypeDef tokTypeDef, 
		WCHAR* szTypeName, ULONG nMaxLen, DWORD* pdwFlags, mdToken* ptokBaseType);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();
	HRESULT getProps() const;

private: // NOT IMPLEMENTED
    typeDef_iterator(const typeDef_iterator& it); 
	typeDef_iterator& operator=(const typeDef_iterator& that);

private: // DATA
	mutable WCHAR m_szTypeName[2048]; // 2048 from alink.h
	mutable DWORD m_dwFlags;
	mutable mdToken m_tokBaseType;
};

/////////////////////
// typeRef_iterator
/////////////////////

class typeRef_iterator : public HCORENUM_iterator<mdTypeRef>
{
public:
	typedef HCORENUM_iterator<mdTypeRef> _Mybase;

	typeRef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);

	const WCHAR* getName() const;
	const SStrProp getValue() const;

	static HRESULT getProps(IMetaDataImport* piImport, mdToken tokType, 
		WCHAR* szTypeName, ULONG nMaxLen, mdToken* ptokResolutionScope);

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();

private: // NOT IMPLEMENTED
	typeRef_iterator(const typeRef_iterator& it); 
	typeRef_iterator& operator=(const typeRef_iterator& that);

private: // DATA
	mutable WCHAR m_szTypeName[2048]; // 2048 from alink.h
	mdToken m_tokResolutionScope;
};

///////////////////////
// methodDef_iterator
///////////////////////

class methodDef_iterator : public HCORENUM_iterator<mdMethodDef>
{
public:
	typedef HCORENUM_iterator<mdMethodDef> _Mybase;

    methodDef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);
	methodDef_iterator(typeDef_iterator& itTypeDef);

	const CMethodInfo& getValue() const;
	CMethodInfo& getValue();
	const WCHAR* getParentName() const;
	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();

private: // NOT IMPLEMENTED
    methodDef_iterator(const methodDef_iterator& it); 
	methodDef_iterator& operator=(const methodDef_iterator& that);

private: // DATA
	CMethodInfo m_methodInfo;
};

/////////////////
// SMemberProps
/////////////////

struct SMemberProps
{
	WCHAR name[2048]; // 2048 from alink.h
	WCHAR fullName[2048*2];
	PCCOR_SIGNATURE sig;
	ULONG siglen;
	IMetaDataImport* piImport;

public:
	
	SMemberProps::SMemberProps()
		: name()
		, fullName()
		, sig(0)
		, siglen(0)
	{ 
		name[0] = L'\0';
		fullName[0] = L'\0';
	}

	const WCHAR* toString() const
	{
		return fullName;
	}

	const WCHAR* toDump() const
	{
		static WCHAR buf[1000];
		swprintf(buf, L"(%d) ", siglen);

		for(unsigned int i = 0; i < siglen; ++i)
		{
			WCHAR byteStr[10];
			BYTE b = *(sig+i);
			swprintf(byteStr, L"%02X", b);
			wcscat(buf, byteStr);
		}

		return buf;
	}

	friend bool operator<(const SMemberProps& lhs, const SMemberProps& rhs)
	{
		int cmp = wcscmp(lhs.fullName, rhs.fullName);
		if (cmp != 0)
			return cmp < 0;

		return SigCmp(	lhs.sig, lhs.siglen, lhs.piImport, 
						rhs.sig, rhs.siglen, rhs.piImport,
						NULL, NULL) < 0;
	}
};

///////////////////////
// memberRef_iterator
///////////////////////

class memberRef_iterator : public HCORENUM_iterator<mdMemberRef>
{
public:
	typedef HCORENUM_iterator<mdMemberRef> _Mybase;

	memberRef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);
	memberRef_iterator(typeRef_iterator& itTypeRef);

	const SMemberProps& getValue() const;
	const WCHAR* getName() const;
	const WCHAR* getParentName() const;
	const WCHAR* getFullName() const;

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();
	HRESULT memberRef_iterator::getProps(mdMemberRef tokMemberRef, WCHAR* szMemberName, ULONG nMaxLen, 
		WCHAR* szFullName, ULONG /*nMaxFullNameLen*/,
		const mdTypeRef& tokEnclosingTypeRef, PCCOR_SIGNATURE* ppSignature, 
		ULONG* pnSignatureLength) const;

private: // NOT IMPLEMENTED
	memberRef_iterator(const memberRef_iterator& it); 
	memberRef_iterator& operator=(const memberRef_iterator& that);

private: // DATA
	SMemberProps m_props;
};

/////////////////
// SMemberProps
/////////////////

struct SFieldProps
{
	IMetaDataImport* piImport;
	WCHAR name[2048]; // 2048 from alink.h
	WCHAR fullName[2048*2];
	DWORD attrs;
	PCCOR_SIGNATURE sig;
	ULONG siglen;
	DWORD cplus;
	const void* val;
	ULONG valsize;

public:

	SFieldProps::SFieldProps()
		: piImport(0)
		, name()
		, fullName()
		, attrs(0)
		, sig(0)
		, siglen(0)
		, cplus(0)
		, val(0)
		, valsize(0)
	{ 
		name[0] = L'\0';
		fullName[0] = L'\0';
	}

	const WCHAR* toString() const
	{
		return fullName;
	}

	const WCHAR* toDump() const
	{
		return 0;
	}

	friend bool operator<(const SFieldProps& lhs, const SFieldProps& rhs)
	{
		int cmp = wcscmp(lhs.name, rhs.name);
		if (cmp != 0)
			return cmp < 0;
		//return SigCmp(	lhs.sig, lhs.siglen, lhs.piImport, 
		//	rhs.sig, rhs.siglen, rhs.piImport,
		//	NULL, NULL) < 0;
		return cmp < 0;
	}
};

///////////////////////
// fieldDef_iterator
///////////////////////

class fieldDef_iterator : public HCORENUM_iterator<mdFieldDef>
{
public:
	typedef HCORENUM_iterator<mdFieldDef> _Mybase;

	fieldDef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);
	fieldDef_iterator(typeRef_iterator& itTypeRef);

	const SFieldProps& getValue() const;
	const WCHAR* getName() const;
	const WCHAR* getParentName() const;
	const WCHAR* getFullName() const;

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();

private: // NOT IMPLEMENTED
	fieldDef_iterator(const fieldDef_iterator& it); 
	fieldDef_iterator& operator=(const fieldDef_iterator& that);

private: // DATA
	SFieldProps m_props;
};

//////////////////////
// paramDef_iterator
//////////////////////

class paramDef_iterator : public HCORENUM_iterator<mdParamDef>
{
public:
	typedef HCORENUM_iterator<mdParamDef> _Mybase;

    paramDef_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);
	paramDef_iterator(methodDef_iterator& itMethodDef);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();

private: // NOT IMPLEMENTED
    paramDef_iterator(const paramDef_iterator& it); 
	paramDef_iterator& operator=(const paramDef_iterator& that);
};

struct SStrWLen
{
	std::wstring str;
	ULONG len;

	const WCHAR* toString() const
	{
		return str.c_str();
	}

	const WCHAR* toDump() const
	{
		return 0;
	}

	friend bool operator<(const SStrWLen& lhs, const SStrWLen& rhs)
	{
		int nCmp = lhs.str.compare(rhs.str);
		return nCmp < 0 || (nCmp == 0 && lhs.len < rhs.len);
	}
};

// This method is needed by map<SStrWLen, mdToken> to properly sort
bool std::less<SStrWLen>::operator()(const SStrWLen& lhs, const SStrWLen& rhs) const
{
	return lhs < rhs;
}

////////////////////////
// userString_iterator
////////////////////////

class userString_iterator : public HCORENUM_iterator<mdString>
{
public:
	typedef HCORENUM_iterator<mdString> _Mybase;

	// Needed by MetadataDiff template function
	userString_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);

	~userString_iterator();

	// The string returned will be invalid if you increment or
	// destroy the iterator
	const SStrWLen& getValue() const;

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void reserve(ULONG nCapacity) const;

private: // NOT IMPLEMENTED
    userString_iterator(const userString_iterator& it); 
	userString_iterator& operator=(const userString_iterator& that);

private:
	mutable WCHAR* m_pszTemp;
	mutable ULONG  m_ulSize;

	static const ULONG nINITIAL_SIZE	= 50;
	static const ULONG nINC_FACTOR		= 2;
};

//////////////////////
// SSigProps
//////////////////////
struct SSigProps
{
	IMetaDataImport* piImport;
	PCCOR_SIGNATURE sig;
	ULONG len;
	WCHAR sz[2048];

	SSigProps::SSigProps()
		: piImport()
		, sig()
		, len(0)
		, sz()
	{ 
		sz[0] = L'\0';
	}

	void init()
	{
		_ASSERTE(piImport);

		if (sig)
		{
			Sig2String(piImport, sig, len, sz, (sizeof(sz)/sizeof(*sz))-1);
		}
	}

	const WCHAR* toString() const
	{
		_ASSERTE(piImport);
		//_ASSERTE(init has been called);

		return sz;
	}

	const WCHAR* toDump() const
	{
		return 0;
	}

	friend bool operator==(const SSigProps& lhs, const SSigProps& rhs)
	{
		_ASSERTE(lhs.piImport && rhs.piImport);
		return wcscmp(lhs.sz, rhs.sz) == 0;
	}

	friend bool operator<(const SSigProps& lhs, const SSigProps& rhs)
	{
		_ASSERTE(lhs.piImport && rhs.piImport);
		return wcscmp(lhs.sz, rhs.sz) < 0;
	}
};

//////////////////////
// signature_iterator
//////////////////////

class signature_iterator : public HCORENUM_iterator<mdSignature>
{
public:
	typedef HCORENUM_iterator<mdSignature> _Mybase;

    signature_iterator(IMetaDataImport* piImport, LPCBYTE pbLoadAddress, bool bIsImage, mdToken tokParent);

	const SSigProps& getValue();

	HRESULT emit(IMetaDataEmit* piMetaDataEmit, mdToken tokEnclosingScope, mdToken* ptokEmitted);

private: // HELPER FUNCTIONS
	void getNextBatchImpl();
	void initCurrent();

private: // NOT IMPLEMENTED
    signature_iterator(const signature_iterator& it); 
	signature_iterator& operator=(const signature_iterator& that);

private: // DATA
	SSigProps m_props;
};

#endif // INCLUDE_METADATA_ITERATORS
