///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Patcher/ModuleInfo.cpp 5     5/28/05 1:20p Eaddy $
//
//    File name:	ModuleInfo.cpp
//   Created By:	Marc Eaddy
//   Created on:	10/10/04
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Patcher/ModuleInfo.cpp $
 * 
 * 5     5/28/05 1:20p Eaddy
 * Changed copyright
 * Revision 1.5  2004/10/24 19:33:11  me133
 * o Implemented heartbeat message
 * o Created WatcherKeeper thread for starting and stopping the Watcher
 * and for sending the heartbeat message
 * o Now reads Watcher path from env variable
 * o Fixed bug where meerly changing a string didn't cause a change to be
 * detected
 *
 * Revision 1.4  2004/10/17 08:46:52  me133
 * o Now uses the incremental file info to pass the correct compile settings
 * to Watcher. Still need to pass resource files.  We aren't able to parse
 * the incr file in all cases (Text3D.exe.incr built with /debug).
 *
 * Revision 1.3  2004/10/17 00:24:55  me133
 * o Now provides a class-based iface to get to the incremental file info
 *
 * Revision 1.2  2004/10/10 06:07:45  me133
 * o Added header comment
 */

#include "StdAfx.h"

#include "ModuleInfo.h"		// Me

#include "DebugHelper.h"	// SDebugHelper::GetSourceFilesForModule()

///////////////////////////////
// MODULE INFO IMPLEMENTATION
///////////////////////////////

CModuleInfo::CModuleInfo()
	: m_moduleID(0)
	, m_strModulePath()
	, m_pbBaseLoadAddress(0)
	, m_strTmpPath()
	, m_bGotTmpPath(false)
	, m_pCompileInfo(0)
	, m_incrBuildFile()
	, m_bugReportFile()
{ }

CModuleInfo::~CModuleInfo()
{ }

HRESULT CModuleInfo::init(	const ModuleID& moduleID,
							const WCHAR* szModulePath,
							LPCBYTE pbBaseLoadAddress)
{
	_ASSERTE(moduleID != 0 && szModulePath && *szModulePath && pbBaseLoadAddress);
	if (moduleID == 0 || !szModulePath || !*szModulePath || !pbBaseLoadAddress)
		return E_POINTER;

	m_moduleID			= moduleID;
	m_strModulePath		= szModulePath;
	m_pbBaseLoadAddress = pbBaseLoadAddress;

	m_strTmpPath.clear();
	m_bGotTmpPath = false;

	return S_OK;
}

ModuleID CModuleInfo::getModuleID() const
{
	return m_moduleID;
}

const WCHAR* CModuleInfo::getModulePath() const
{
	return m_strModulePath.c_str();
}

LPCBYTE CModuleInfo::getLoadAddr() const
{
	return m_pbBaseLoadAddress;
}

HRESULT CModuleInfo::findAndLoadCompileInfo()
{
	HRESULT hr = S_OK;

	hr = m_bugReportFile.findAndLoad(m_strModulePath.c_str());
	if (SUCCEEDED(hr))
	{
		m_pCompileInfo = &m_bugReportFile;
		return hr;
	}

	hr = m_incrBuildFile.findAndLoad(m_strModulePath.c_str());
	if (SUCCEEDED(hr))
	{
		m_pCompileInfo = &m_incrBuildFile;
		return hr;
	}

	return hr;
}

////////////////////////////////
// MODULE QUEUE IMPLEMENTATION
////////////////////////////////

CModuleQueue::CModuleQueue(const char* szEventName)
	: m_cs()
	, m_hQueuedObjectAvailable()
	, m_queModules()
	, m_szEventName()
{ 
	_ASSERTE(szEventName && *szEventName);
	strcpy(m_szEventName, szEventName);

	::InitializeCriticalSection(&m_cs);

	m_hQueuedObjectAvailable = ::CreateEvent(	0,
												TRUE,
												FALSE,
												m_szEventName);
	if (!m_hQueuedObjectAvailable)
	{
		ILXLOG(HRESULT_FROM_WIN32(::GetLastError()),
			_T("CreateEvent() failed."), ILXLOG_ERROR);
		_ASSERTE(m_hQueuedObjectAvailable);
	}
}

CModuleQueue::~CModuleQueue()
{
	{
		// Lock this block (curly braces are necessary here!)
		CCritSecLock autoCritSec(m_cs);

		while (m_queModules.size() > 0)
		{
			CModuleInfo* pModuleInfo = m_queModules.front();
			delete pModuleInfo;
			m_queModules.pop();
		}
	}

	::CloseHandle(m_hQueuedObjectAvailable);
	::DeleteCriticalSection(&m_cs);
}

void CModuleQueue::push(	const ModuleID& moduleID,
							const WCHAR* szModulePath,
							LPCBYTE pbBaseLoadAddress)
{
	// Gets deleted in DTOR or by the client if the client calls pop()
	CModuleInfo* pModuleInfo = new CModuleInfo();
	pModuleInfo->init(moduleID, szModulePath, pbBaseLoadAddress);
	push(pModuleInfo);
}

void CModuleQueue::push(CModuleInfo* pModuleInfo)
{
	{
		// Lock this block (curly braces are necessary here!)
		CCritSecLock autoCritSec(m_cs);
		m_queModules.push(pModuleInfo);
	}

	BOOL bRet = ::SetEvent(m_hQueuedObjectAvailable);
	if (!bRet)
	{
		ILXLOG(HRESULT_FROM_WIN32(::GetLastError()), _T("SetEvent() failed."), ILXLOG_ERROR);
	}
}

bool CModuleQueue::pop(CModuleInfo** ppModuleInfo)
{
	_ASSERTE(ppModuleInfo);
	if (!ppModuleInfo) return false;

	_ASSERTE(!*ppModuleInfo); // Forget to delete?

	// Acquire critical section which is active until return is called
	CCritSecLock autoCritSec(m_cs);

	size_t nNumItems = m_queModules.size();

	if (nNumItems > 0)
	{
		*ppModuleInfo = m_queModules.front();
		m_queModules.pop();

		// Did we remove the last item?
		if (nNumItems == 1)
		{
			::ResetEvent(m_hQueuedObjectAvailable);
		}

		return true;
	}
	else
	{
		//_ASSERTE(m_hQueuedObjectAvailable is not set);
		return false;
	}
}

HANDLE CModuleQueue::getEvent() const
{
	return m_hQueuedObjectAvailable;
}
