// Note: Some function header comments came from corprof.idl - Marc Eaddy

#include "StdAfx.h"

#include "PatcherProfilerCallback.h"	// Me

#include "IlxLogHelper.h"				// Logging

/*
 *
 * JIT EVENTS
 *
 */

/*
 * The CLR calls FunctionUnloadStarted to notify the code
 * profiler that a function is being unloaded.  After returning from this
 * call, the FunctionID is no longer valid.
 *
 * NOTE: In the current version of the runtime, this callback never occurs.
 */
STDMETHODIMP CPatcherProfilerCallback::FunctionUnloadStarted(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("FunctionUnloadStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls JITCompilationFinished to notify the code
 * profiler that the JIT compiler has finished compiling a function.
 *
 * The fIsSafeToBlock argument tells the profiler whether or not blocking
 * will affect the operation of the runtime.  If true, blocking may cause
 * the runtime to wait for the calling thread to return from this callback.
 * Although this will not harm the runtime, it will skew the profiling
 * results.
 *
 * The FunctionID is now valid in ICorProfilerInfo APIs.
 *
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::JITCompilationFinished(
            /*[in]*/ FunctionID functionId,
            /*[in]*/ HRESULT    hrStatus,
            /*[in]*/ BOOL       fIsSafeToBlock)
{
	//ILXLOGMSG0(ILXLOG_INFO, _T("JITCompilationFinished() called."));
	return S_OK;
}

/*
 * This notifies the profiler when a search for a prejitted function is
 * starting.
 *
 *    functionId: the function for which the search is being performed.
 *    bUseCachedFunction: if true, the EE uses the cached function (if applicable)
 *                        if false, the EE jits the function instead of
 *                        using a pre-jitted version.
 */
STDMETHODIMP CPatcherProfilerCallback::JITCachedFunctionSearchStarted(
            /*[in]*/ FunctionID functionId,
            /*[out]*/ BOOL* pbUseCachedFunction)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("JITCachedFunctionSearchStarted() called."));
	return S_OK;
}

/*
 * This notifies the profiler when a search for a cached function has been
 * performed.
 *
 *    functionId: the function for which the search has been performed.
 *    result: the result of the search.  There are two possible results:
 *        COR_PRF_CACHED_FUNCTION_FOUND
 *        COR_PRF_CACHED_FUNCTION_NOT_FOUND
 *
 */
STDMETHODIMP CPatcherProfilerCallback::JITCachedFunctionSearchFinished(
            /*[in]*/ FunctionID        functionId,
            /*[in]*/ COR_PRF_JIT_CACHE result)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("JITCachedFunctionSearchFinished() called."));
	return S_OK;
}

/*
 * The CLR calls JITFunctionPitched to notify the profiler
 * that a jitted function was removed from memory.  If the pitched
 * function is called in the future, the profiler will receive new
 * JIT compilation events as it is re-jitted.
 *
 * NOTE: the FunctionID is not valid until it is re-jitted.  When it is
 * re-jitted, it will use the same FunctionID value.
 */
STDMETHODIMP CPatcherProfilerCallback::JITFunctionPitched(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("JITFunctionPitched() called."));
	return S_OK;
}

/*
 * The CLR calls JITInlining to notify the profiler that the jitter
 * is about to inline calleeId into callerId.  Set pfShouldInline to FALSE
 * to prevent the callee from being inlined into the caller, and set to
 * TRUE to allow the inline to occur.
 *
 * NOTE: Inlined functions do not provide Enter/Leave events, so if you desire
 *       an accurate callgraph, you should set FALSE.  Be aware that
 *       setting FALSE will affect performance, since inlining typically
 *       increases speed and reduces separate jitting events for the inlined
 *       method.
 */
STDMETHODIMP CPatcherProfilerCallback::JITInlining(
            /*[in]*/ FunctionID callerId,
            /*[in]*/ FunctionID calleeId,
            /*[out]*/ BOOL* pfShouldInline)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("JITInlining() called."));
	return S_OK;
}

/*
 *
 * APPLICATION DOMAIN EVENTS
 *
 */

/*
 * Called when an application domain creation has begun and ended.
 * The id is not valid for any information request until after the app
 * domain has been fully created.  One may only cache the id provided in
 * AppDomainCreationStarted for later use.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::AppDomainCreationStarted(
            /*[in]*/ AppDomainID appDomainId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AppDomainCreationStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::AppDomainCreationFinished(
            /*[in]*/ AppDomainID appDomainId,
            /*[in]*/ HRESULT     hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AppDomainCreationFinished() called."));
	return E_NOTIMPL;
}

/*
 * Called before and after an app domain is unloaded from a process.
 * Once an AppDomain unload has started, its ID is no longer valid in the
 * system and cannot be used in calls to any ICorProfilerInfo APIs.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::AppDomainShutdownStarted(
            /*[in]*/ AppDomainID appDomainId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AppDomainShutdownStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::AppDomainShutdownFinished(
            /*[in]*/ AppDomainID appDomainId,
            /*[in]*/ HRESULT     hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AppDomainShutdownFinished() called."));
	return E_NOTIMPL;
}

/*
 *
 * ASSEMBLY EVENTS
 *
 */

/*
 * Called when an Assembly load has begun and ended. The id is not valid
 * for any information request until after the assembly has been fully
 * loaded.  One may only cache the id provided in AssemblyLoadStarted for
 * later use.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::AssemblyLoadStarted(
            /*[in]*/ AssemblyID assemblyId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AssemblyLoadStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::AssemblyLoadFinished(
            /*[in]*/ AssemblyID assemblyId,
            /*[in]*/ HRESULT    hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AssemblyLoadFinished() called."));
	return E_NOTIMPL;
}

/*
 * Called before and after an assembly is unloaded.  AssemblyUnloadStarted
	* is the last point at which the AssemblyID is valid for calls to the
	* ICorProfilerInfo interface.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::AssemblyUnloadStarted(
            /*[in]*/ AssemblyID assemblyId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AssemblyUnloadStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::AssemblyUnloadFinished(
            /*[in]*/ AssemblyID assemblyId,
            /*[in]*/ HRESULT    hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("AssemblyUnloadFinished() called."));
	return E_NOTIMPL;
}

/*
 *
 * MODULE EVENTS
 *
 */

/*
 * The CLR calls ModuleLoadStarted to notify the code
 * profiler that a module is about to be loaded.  Because this module has not
 * yet been loaded, other module information methods are not valid to
 * call in this callback.  Wait until ModuleLoadFinished() is
 * received to do things like acquire the metadata.
 */
STDMETHODIMP CPatcherProfilerCallback::ModuleLoadStarted(
            /*[in]*/ ModuleID moduleId)
{
	//ILXLOGMSG0(ILXLOG_INFO, _T("ModuleLoadStarted() called."));
	return E_NOTIMPL;
}

/*
 * Called before a module is being unloaded.  Use this events to collect
 * final statics that require the ModuleID to be valid.  After returning
 * from ModuleUnloadStarted, the ModuleID is no longer valid.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::ModuleUnloadStarted(
            /*[in]*/ ModuleID moduleId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ModuleUnloadStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::ModuleUnloadFinished(
            /*[in]*/ ModuleID moduleId,
            /*[in]*/ HRESULT  hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ModuleUnloadFinished() called."));
	return E_NOTIMPL;
}

/*
 * A module can get loaded through legacy means (ie: IAT or LoadLibrary) or
 * through a metadata reference.  The CLR loader therefore has many code
 * paths for determining what assembly a module lives in.  It is therefore
 * possible that after a ModuleLoadFinished event, the module does not
 * know what assembly it is in and getting the parent AssemblyID is not possible.
 * This event is fired when the module is officially attached to its parent
 * assembly.  Calling GetModuleInfo after this point will return the proper parent
 * assembly.
 */
STDMETHODIMP CPatcherProfilerCallback::ModuleAttachedToAssembly(
            /*[in]*/ ModuleID   moduleId,
            /*[in]*/ AssemblyID AssemblyId)
{
	//ILXLOGMSG0(ILXLOG_INFO, _T("ModuleAttachedToAssembly() called."));
	return E_NOTIMPL;
}

/*
 *
 *  CLASS EVENTS
 *
 */

/*
 * The CLR calls ClassLoadStarted to notify the code profiler
 * that a class is being loaded.  The ClassID is not valid for calls to
 * the ICorProfilerInfo interface until the profiler receives a
 * ClassLoadFinished event for the same ClassID.
 */
STDMETHODIMP CPatcherProfilerCallback::ClassLoadStarted(
            /*[in]*/ ClassID classId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ClassLoadStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ClassLoadFinished to notify the code
 * profiler that a class has been loaded.  The ClassID is now valid for
 * calls to the ICorProfilerInfo interface.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::ClassLoadFinished(
            /*[in]*/ ClassID classId,
            /*[in]*/ HRESULT hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ClassLoadFinished() called."));
	return E_NOTIMPL;
}

/*
 * The given class is about to be unloaded.  Use this event to gather final
 * status and clean up anything that requires the ClassID to be valid.
 * After returning from this callback the ClassID is no longer valid.
 * The hrStatus provides the success or failure of the operation
 */
STDMETHODIMP CPatcherProfilerCallback::ClassUnloadStarted(
            /*[in]*/ ClassID classId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ClassUnloadStarted() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::ClassUnloadFinished(
            /*[in]*/ ClassID classId,
            /*[in]*/ HRESULT hrStatus)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ClassUnloadFinished() called."));
	return E_NOTIMPL;
}

/*
 *
 * THREAD EVENTS
 *
 */

/*
 * The CLR calls ThreadCreated to notify the code profiler
 * that a thread has been created.  The ThreadID is valid immediately.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::ThreadCreated(
            /*[in]*/ ThreadID threadId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ThreadCreated() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ThreadDestroyed to notify the code profiler
 * that a thread has been destroyed.  The ThreadID is no longer valid.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::ThreadDestroyed(
            /*[in]*/ ThreadID threadId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ThreadDestroyed() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ThreadAssignedToOSThread to tell the profiler
 * that a managed thread is being implemented via a particualr OS thread.
 * This callback exists so that the profiler can maintain an accurate
 * OS to Managed thread mapping across fibres.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::ThreadAssignedToOSThread(
            /*[in]*/ ThreadID managedThreadId,
            /*[in]*/ DWORD    dwOsThreadId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ThreadAssignedToOSThread() called."));
	return E_NOTIMPL;
}

/*
 *
 * REMOTING EVENTS
 *
 *
 * NOTE: Inprocess debugging not supported from these callbacks.
 */

//
// Client-side events
//

/*
 * NOTE: each of the following pairs of callbacks will occur on the same
 *       thread
 *   RemotingClientInvocationStarted  & RemotingClientSendingMessage
 *   RemotingClientReceivingReply     & RemotingClientInvocationFinished
 *   RemotingServerInvocationReturned & RemotingServerSendingReply
 */
/*
 * The CLR calls RemotingClientInvocationStarted to notify the profiler that
 * a remoting call has begun.  This event is the same for synchronous and
 * asynchronous calls.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingClientInvocationStarted()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingClientInvocationStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingClientSendingMessage to notify the profiler that
 * a remoting call is requiring the the caller to send an invocation request through
 * a remoting channel.
 *
 * pCookie  - if remoting GUID cookies are active, this value will correspond with the
 *            the value provided in RemotingServerReceivingMessage, if the channel
 *            succeeds in transmitting the message, and if GUID cookies are active on
 *            the server-side process.  This allows easy pairing of remoting calls,
 *            and the creation of a logical call stack.
 * fIsAsync - is true if the call is asynchronous.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingClientSendingMessage(
            /*[in]*/ GUID* pguidCookie,
            /*[in]*/ BOOL fIsAsync)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingClientSendingMessage() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingClientReceivingReply to notify the profiler that
 * the server-side portion of a remoting call has completed and that the client is
 * now receiving and about to process the reply.
 *
 * pCookie  - if remoting GUID cookies are active, this value will correspond with the
 *            the value provided in RemotingServerSendingReply, if the channel
 *            succeeds in transmitting the message, and if GUID cookies are active on
 *            the server-side process.  This allows easy pairing of remoting calls.
 * fIsAsync - is true if the call is asynchronous.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingClientReceivingReply(
            /*[in]*/ GUID* pguidCookie,
            /*[in]*/ BOOL fIsAsync)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingClientReceivingReply() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingClientInvocationFinished to notify the profiler that
 * a remoting invocation has run to completion on the client side.  If the call was
 * synchronous, this means that it has also run to completion on the server side.  If
 * the call was asynchronous, a reply may still be expected when the call is handled.
 * If the call is asynchronous, and a reply is expected, then the reply will occur in
 * the form of a call to RemotingClientReceivingReply and an additional call to
 * RemotingClientInvocationFinished to indicate the required secondary processing of
 * an asynchronous call.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingClientInvocationFinished()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingClientInvocationFinished() called."));
	return E_NOTIMPL;
}

//
// Server-side events
//

/*
 * The CLR calls RemotingServerReceivingMessage to notify the profiler that
 * the process has received a remote method invocation (or activation) request.  If
 * the message request is asynchronous, then the request may be serviced by any
 * arbitrary thread.
 *
 * pCookie  - if remoting GUID cookies are active, this value will correspond with the
 *            the value provided in RemotingClientSendingMessage, if the channel
 *            succeeds in transmitting the message, and if GUID cookies are active on
 *            the client-side process.  This allows easy pairing of remoting calls.
 * fIsAsync - is true if the call is asynchronous.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingServerReceivingMessage(
            /*[in]*/ GUID* pguidCookie,
            /*[in]*/ BOOL fIsAsync)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingServerReceivingMessage() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingServerInvocationStarted to notify the profiler that
 * the process is invoking a method due to a remote method invocation request.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingServerInvocationStarted()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingServerInvocationStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingServerInvocationReturned to notify the profiler that
 * the process has finished invoking a method due to a remote method invocation request.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingServerInvocationReturned()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingServerInvocationReturned() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RemotingServerSendingReply to notify the profiler that
 * the process has finished processing a remote method invocation request and is
 * about to transmit the reply through a channel.
 *
 * pCookie  - if remoting GUID cookies are active, this value will correspond with the
 *            the value provided in RemotingClientReceivingReply, if the channel
 *            succeeds in transmitting the message, and if GUID cookies are active on
 *            the client-side process.  This allows easy pairing of remoting calls.
 * fIsAsync - is true if the call is asynchronous.
 */
STDMETHODIMP CPatcherProfilerCallback::RemotingServerSendingReply(
            /*[in]*/ GUID* pguidCookie,
            /*[in]*/ BOOL fIsAsync)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RemotingServerSendingReply() called."));
	return E_NOTIMPL;
}

/*
 *
 * TRANSITION EVENTS
 *
 */

/*
 * The CLR calls UnmanagedToManagedTransition to notify the
 * code profiler that a transition from unmanaged code to managed code has
 * occurred. functionId is always the ID of the callee, and reason
 * indicates whether the transition was due to a call into managed code from
 * unmanaged, or a return from an unmanaged function called by a managed one.
 *
 * Note that if the reason is COR_PRF_TRANSITION_RETURN, then the functionId
 * is that of the unmanaged function, and will never have been jitted.
 * Unmanaged functions still have some basic information associated with
 * them, such as a name, and some metadata.
	*
	* Note that if the reason is COR_PRF_TRANSITION_RETURN and the callee was
	* a PInvoke call indirect, then the runtime does not know the destination
	* of the call and functionId will be NULL.
	*
	* Note that if the reason is COR_PRF_TRANSITION_CALL then it may be possible
	* that the callee has not yet been JIT-compiled.
 */
STDMETHODIMP CPatcherProfilerCallback::UnmanagedToManagedTransition(
            /*[in]*/ FunctionID functionId,
            /*[in]*/ COR_PRF_TRANSITION_REASON reason)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("UnmanagedToManagedTransition() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ManagedToUnmanagedTransition to notify the
 * code profiler that a transition from managed code to unmanaged code has
 * occurred. functionId is always the ID of the callee, and reason
 * indicates whether the transition was due to a call into unmanaged code from
 * managed, or a return from an managed function called by an unmanaged one.
 *
 * Note that if the reason is COR_PRF_TRANSITION_CALL, then the functionId
 * is that of the unmanaged function, and will never have been jitted.
 * Unmanaged functions still have some basic information associated with
 * them, such as a name, and some metadata.
	*
	* Note that if the reason is COR_PRF_TRANSITION_CALL and the callee is
	* a PInvoke call indirect, then the runtime does not know the destination
	* of the call and functionId will be NULL.
 */
STDMETHODIMP CPatcherProfilerCallback::ManagedToUnmanagedTransition(
            /*[in]*/ FunctionID                functionId,
            /*[in]*/ COR_PRF_TRANSITION_REASON reason)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ManagedToUnmanagedTransition() called."));
	return E_NOTIMPL;
}

/*
 *
 * RUNTIME SUSPENSION EVENTS
 *
 */

/*
 * NOTE: None of the suspension callbacks are supported by in-process
 *       debugging *except* ObjectReferences.
 */

/*
 * The CLR calls RuntimeSuspendStarted to notify the code profiler
 * that the runtime is about to suspend all of the runtime threads.
 * All runtime threads that are in unmanaged code are permitted to continue
 * running until they try to re-enter the runtime, at which point they will
 * also suspend until the runtime resumes.  This also applies to new threads
 * that enter the runtime.  All threads within the runtime are either
 * suspended immediately if they are in interruptible code, or asked to
 * suspend when they do reach interruptible code.
 *
 * suspendReason make be any of the following values:
 *  COR_PRF_SUSPEND_FOR_GC
 *      the runtime is suspending to service a GC request.  The GC-related
 *      callbacks will occur between the RuntimeSuspendFinished and
 *      RuntimeResumeStarted events.
 *  COR_PRF_SUSPEND_FOR_CODE_PITCHING
 *      the runtime is suspending so that code pitching may occur.  This
 *      only occurs when the EJit is active with code pitching enabled.
 *      Code pitching callbacks will occur between the
 *      RuntimeSuspendFinished and RuntimeResumeStarted events.
 *  COR_PRF_SUSPEND_FOR_APPDOMAIN_SHUTDOWN
 *      the runtime is suspending so that an AppDomain can be shut down.
 *      While the runtime is suspended, the runtime will determine which
 *      threads are in the AppDomain that is being shut down, set them to
 *      abort when they resume, and then resumes the runtime.  There are
 *      no AppDomain-specific callbacks during this suspension.
 *  COR_PRF_SUSPEND_FOR_SHUTDOWN
 *      the runtime is shutting down, and it must suspend all threads to
 *      complete the operation.
 *  COR_PRF_SUSPEND_FOR_GC_PREP
 *      the runtime is preparing for a GC.
 *  COR_PRF_SUSPEND_FOR_INPROC_DEBUGGER
 *      the runtime is suspending for in-process debugging.
 *  COR_PRF_SUSPEND_OTHER
 *      the runtime is suspending for a reason other than those above.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeSuspendStarted(
        /*[in]*/ COR_PRF_SUSPEND_REASON suspendReason)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeSuspendStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls SyncForSuspendFinished to notify the code profiler
 * that the runtime has suspended all threads needed for a runtime
 * suspension.  Note that not all runtime threads are required to be
 * suspended, as described in the comment for SyncForSuspendStarted.
 *
 * NOTE: It is guaranteed that this event will occur on the same ThreadID
 * as RuntimeSuspendStarted occurred on.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeSuspendFinished()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeSuspendFinished() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RuntimeSuspendAborted to notify the code profiler
 * that the runtime is aborting the runtime suspension that was occurring.
 * This may occur if two threads simultaneously attempt to suspend the
 * runtime.
 *
 * NOTE: It is guaranteed that this event will occur on the same ThreadID
 * as the RuntimeSuspendStarted occurred on, and that only one of
 * RuntimeSuspendFinished and RuntimeSuspendAborted may occur on a single
 * thread following a RuntimeSuspendStarted event.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeSuspendAborted()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeSuspendAborted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RuntimeResumeStarted to notify the code profiler
 * that the runtime is about to resume all of the runtime threads.
 *
 * NOTE: If a thread successfully suspended the runtime, then it is *NOT*
 * guaranteed that the call to RuntimeResumeStarted will be called on the
 * same ThreadID.  In this case, it is also not guaranteed that the thread
 * calling RuntimeResumeStarted will have a non-zero ThreadID.
 * However, if a thread unsuccessfully suspends the runtime
 * (i.e., there was a conflict with another suspension), then it *IS*
 * guaranteed that the calls to RuntimeSuspspendStarted,
 * RuntimeSuspendAborted and RuntimeResumeStarted will be called from the
 * same ThreadID.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeResumeStarted()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeResumeStarted() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RuntimeResumeFinished to notify the code profiler
 * that the runtime has finished resuming all of it's threads and is now
 * back in normal operation.
 *
 * NOTE: It is *NOT* guaranteed that this event will occur on the same
 * ThreadID as the RuntimeSuspendStarted occurred on, but is guaranteed
 * to occur on the same ThreadID as the RuntimeResumeStarted occurred on.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeResumeFinished()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeResumeFinished() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ThreadSuspended to notify the code profiler
 * that a particular thread has been suspended.  All threads within managed
 * code must be suspended.  If a thread is in unmanaged code, it will be
 * allowed to continue, but will suspend upon re-entering the runtime
 * and will fire this event.  Thus, this notification could occur after
 * a suspension has completed, but before the runtime resumes.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeThreadSuspended(
                /*[in]*/ ThreadID threadId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeThreadSuspended() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ThreadResumed to notify the code profiler
 * that a particular thread has been resumed after being suspended due to
 * a runtime suspension.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RuntimeThreadResumed(
                /*[in]*/ ThreadID threadId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RuntimeThreadResumed() called."));
	return E_NOTIMPL;
}

/*
 *
 * GC EVENTS
 *
 */

/*
 * The CLR calls MovedReferences with information about
 * object references that moved as a result of garbage collection.
 *
 * cMovedObjectIDRanges is a count of the number of ObjectID ranges that
 *      were moved.
 * oldObjectIDRangeStart is an array of elements, each of which is the start
 *      value of a range of ObjectID values before being moved.
 * newObjectIDRangeStart is an array of elements, each of which is the start
 *      value of a range of ObjectID values after being moved.
 * cObjectIDRangeLength is an array of elements, each of which states the
 *      size of the moved ObjectID value range.
 *
 * The last three arguments of this function are parallel arrays.
 *
 * In other words, if an ObjectID value lies within the range
 *      oldObjectIDRangeStart[i] <= ObjectID < oldObjectIDRangeStart[i] + cObjectIDRangeLength[i] 
 * for 0 <= i < cMovedObjectIDRanges, then the ObjectID value has changed to
 *      ObjectID - oldObjectIDRangeStart[i] + newObjectIDRangeStart[i] 
 *
 * NOTE: All of these callbacks are made while the runtime is suspended, so
 *       none of the ObjectID values can change until the runtime resumes
 *       and another GC occurs.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 *
 * NOTE: The profiler will receive GC-related events (except ObjectAllocated)
 *       when the profiler has been suspended for COR_PRF_SUSPEND_FOR_GC *except*
 *       for one special case.  When the runtime is shutting down, there is a
 *       stage where it is suspended for COR_PRF_SUSPEND_FOR_SHUTDOWN reason and
 *       is never resumed.  But after this suspension a garbage collection may
 *       occur without a COR_PRF_SUSPEND_FOR_GC suspension notification, and
 *       the profiler will thus receive GC-related callbacks.
 */
STDMETHODIMP CPatcherProfilerCallback::MovedReferences(
            /*[in]*/                                ULONG    cMovedObjectIDRanges,
            /*[in, size_is(cMovedObjectIDRanges)]*/ ObjectID aryOldObjectIDRangeStart[],
            /*[in, size_is(cMovedObjectIDRanges)]*/ ObjectID aryNewObjectIDRangeStart[],
            /*[in, size_is(cMovedObjectIDRanges)]*/ ULONG    aryObjectIDRangeLength[] )
{
	ILXLOGMSG0(ILXLOG_INFO, _T("MovedReferences() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ObjectAllocated to notify the code profiler
 * an object was allocated on the heap.
 */
STDMETHODIMP CPatcherProfilerCallback::ObjectAllocated(
            /*[in]*/ ObjectID objectId,
            /*[in]*/ ClassID classId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ObjectAllocated() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ObjectsAllocatedByClass to notify the code
 * profiler about the number of objects of a particular class that were
 * allocated since the previous garbage collection. The classes and the
 * counts are passed in parallel arrays.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 *
 * NOTE: This callback will not report objects allocated in the large
 *       object heap.
 */
STDMETHODIMP CPatcherProfilerCallback::ObjectsAllocatedByClass(
            /*[in]*/                       ULONG   cClassCount,
            /*[in, size_is(cClassCount)]*/ ClassID aryClassIds[],
            /*[in, size_is(cClassCount)]*/ ULONG   aryObjects[] )
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ObjectsAllocatedByClass() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ObjectReferences to provide information
 * about objects in memory referenced by a given object.  This function
 * is called for each object remaining in the GC heap after a collection
 * has completed.  If the profiler returns an error from this callback,
 * the profiling services will discontinue invoking this callback until the
 * next GC.  This callback can be used in conjunction with the
 * RootReferences callback to create a complete object reference graph for
 * the runtime.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::ObjectReferences(
            /*[in]*/                       ObjectID objectId,
            /*[in]*/                       ClassID  classId,
            /*[in]*/                       ULONG    cObjectRefs,
            /*[in, size_is(cObjectRefs)]*/ ObjectID aryObjectRefIds[] )
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ObjectReferences() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls RootReferences with information about root
 * references after a garbage collection has occurred. Static object
 * references and references to objects on a stack are co-mingled in the
 * arrays.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::RootReferences(
            /*[in]*/                     ULONG    cRootRefs,
            /*[in, size_is(cRootRefs)]*/ ObjectID aryRootRefIds[] )
{
	ILXLOGMSG0(ILXLOG_INFO, _T("RootReferences() called."));
	return E_NOTIMPL;
}

/*
 *
 * EXCEPTION EVENTS
 *
 *
 * NOTE: Inprocess debugging not supported from any exception callbacks
 *       except ExceptionThrown.
 */

//
// Exception creation
//

/*
 * The CLR calls ExceptionThrown to notify the code
 * profiler that an exception has been thrown.
 *
 * NOTE: This function is only called if the CLR exception handler
 *       is called to process an exception.
 *
 * NOTE: For in-process debugging, thrownObjectId is always NULL.  The
 *       actual thrown object can be found through
 *       ICorDebugThread::GetCurrentException
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionThrown(
            /*[in]*/ ObjectID thrownObjectId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionThrown() called."));
	return E_NOTIMPL;
}

//
// Search phase
//

/*
 * The CLR calls ExceptionSearchFunctionEnter to notify the profiler
 * that the search phase of exception handling has entered a function.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionSearchFunctionEnter(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionSearchFunctionEnter() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionSearchFunctionLeave to notify the profiler
 * that the search phase of exception handling has left a function.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionSearchFunctionLeave()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionSearchFunctionLeave() called."));
	return E_NOTIMPL;
}

/*
 * The CLR will call ExceptionSearchFilterEnter just before excecuting
 * a user filter.  The functionID is that of the function containing the filter.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionSearchFilterEnter(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionSearchFilterEnter() called."));
	return E_NOTIMPL;
}

/*
 * The CLR will call ExceptionSearchFilterLeave immediately after
 * executing a user filter.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionSearchFilterLeave()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionSearchFilterLeave() called."));
	return E_NOTIMPL;
}

/*
 * The CLR will call ExceptionSearchCatcherFound when the search
 * phase of exception handling has located a handler for the exception that
 * was thrown.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionSearchCatcherFound(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionSearchCatcherFound() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionHandlerEnter when the runtime's
 * [Win32 SEH] exception handler is entered.  There may or may not be
 * managed functions guarded by this handler, the profiler may or may not
 * receive exception-related callbacks between ExceptionOSHandlerEnter and
 * ExceptionOSHandlerLeave callbacks.
 *
 * This notification is provided to allow profilers to detect unmanaged-to-
 * managed transitions in stack searches and unwinds.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 *
 * NOTE: NOT YET IMPLEMENTED.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionOSHandlerEnter(
            /*[in]*/ UINT_PTR __unused)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionOSHandlerEnter() called."));
	return E_NOTIMPL;
}

/*
 * This function is similar to ExceptionHandlerEnter, except that
 * it is called just before the runtime's exception handler returns.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 *
 * NOTE: NOT YET IMPLEMENTED.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionOSHandlerLeave(
            /*[in]*/ UINT_PTR __unused)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionOSHandlerLeave() called."));
	return E_NOTIMPL;
}

//
// Unwind phase
//

/*
 * The CLR calls ExceptionUnwindFunctionEnter to notify the profiler
 * that the unwind phase of exception handling has entered a function.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionUnwindFunctionEnter(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionUnwindFunctionEnter() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionUnwindFunctionLeave to notify the profiler
 * that the unwind phase of exception handling has left a function.  The
 * function instance and it's stack data has now been removed from the stack.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionUnwindFunctionLeave()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionUnwindFunctionLeave() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionUnwindFinallyEnter to notify the profiler
 * that the unwind phase of exception is entering a finally clause contained
 * in the specified function.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionUnwindFinallyEnter(
            /*[in]*/ FunctionID functionId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionUnwindFinallyEnter() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionUnwindFinallyLeave to notify the profiler
 * that the unwind phase of exception is leaving a finally clause.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionUnwindFinallyLeave()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionUnwindFinallyLeave() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls this function just before passing control to 
 * the appropriate catch block.  Note that this is called only if the 
 * catch point is in JIT'ed code.  An exception that is caught in 
 * unmanaged code, or in the internal code of the CLR will
 * not generate this notification.  The ObjectID is passed again since
 * a GC could have moved the object since the ExceptionThrown
 * notification.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionCatcherEnter(
            /*[in]*/ FunctionID functionId,
            /*[in]*/ ObjectID   objectId)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionCatcherEnter() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls ExceptionCatcherLeave when the runtime leaves
 * the catcher's code.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionCatcherLeave()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionCatcherLeave() called."));
	return E_NOTIMPL;
}

/*
 *  CLR<->COM interop vtable creation/destruction.
 */

/*
 * The CLR calls this function when an CLR<->COM interop vtable
 * for a particular IID and for a particular class has been created.
 * This provides the ClassID of the class for which this vtable has been
 * created, the IID it implements, the start of the vtable and how many
 * slots are in it.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 *
 * NOTE: It is possible to receive a NULL GUID if the interface is
 *       internal only.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::COMClassicVTableCreated(
            /*[in]*/ ClassID wrappedClassId,
            /*[in]*/ REFGUID implementedIID,
            /*[in]*/ void* pVTable,
            /*[in]*/ ULONG   cSlots)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("COMClassicVTableCreated() called."));
	return E_NOTIMPL;
}

/*
 * The CLR calls this function when a CLR<->COM interop vtable is
 * being destroyed.  Provided are the ClassID, IID and vtable pointer for
 * the destroyed vtable.
 *
 * NOTE: The profiler should not block here, since the stack may not be in a
 *       GC-friendly state and so preemptive GC cannot be enabled.  If the
 *       profiler blocks here and a GC is attempted, the runtime will block
 *       until this callback returns.  Also, the profiler may NOT call into
 *       managed code or in any way cause a managed memory allocation.
 *
 * NOTE: It is possible to receive a NULL GUID if the interface is
 *       internal only.
 *
 * NOTE: Inprocess debugging not supported from this callback.
 */
STDMETHODIMP CPatcherProfilerCallback::COMClassicVTableDestroyed(
            /*[in]*/ ClassID wrappedClassId,
            /*[in]*/ REFGUID implementedIID,
            /*[in]*/ void* pVTable)
{
	ILXLOGMSG0(ILXLOG_INFO, _T("COMClassicVTableDestroyed() called."));
	return E_NOTIMPL;
}

/*
 * When the CLR itself catches an exception, these functions are called
 * to notify the profiler.  Between the Found and Execute callbacks the
 * CLR may also unwind managed frames and so normal ExceptionUnwind*
 * functions can be called in between.
 *
 * NOTE: for backward compatibility the profiler must also set the
 *       COR_PRF_MONITOR_CLR_EXCEPTIONS event flag to receive these events.
 *
 * NOTE: Cannot use inprocess debugging from these callbacks.
 */
STDMETHODIMP CPatcherProfilerCallback::ExceptionCLRCatcherFound()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionCLRCatcherFound() called."));
	return E_NOTIMPL;
}

STDMETHODIMP CPatcherProfilerCallback::ExceptionCLRCatcherExecute()
{
	ILXLOGMSG0(ILXLOG_INFO, _T("ExceptionCLRCatcherExecute() called."));
	return E_NOTIMPL;
}
