///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Patcher/WatcherSocket.cpp 12    12/15/05 2:37a Eaddy $
//
//    File name:	WatcherSocket.cpp
//   Created By:	Marc Eaddy
//   Created on:	10/10/04
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Patcher/WatcherSocket.cpp $
 * 
 * 12    12/15/05 2:37a Eaddy
 * o Improved statistics gathering and display
 * 
 * 11    12/10/05 12:42a Eaddy
 * o Now keeps track of elapsed times
 * 
 * 10    5/28/05 1:20p Eaddy
 * Changed copyright
 * Revision 1.5  2004/10/24 19:33:11  me133
 * o Implemented heartbeat message
 * o Created WatcherKeeper thread for starting and stopping the Watcher
 * and for sending the heartbeat message
 * o Now reads Watcher path from env variable
 * o Fixed bug where meerly changing a string didn't cause a change to be
 * detected
 *
 * Revision 1.4  2004/10/17 08:46:53  me133
 * o Now uses the incremental file info to pass the correct compile settings
 * to Watcher. Still need to pass resource files.  We aren't able to parse
 * the incr file in all cases (Text3D.exe.incr built with /debug).
 *
 * Revision 1.3  2004/10/17 00:24:55  me133
 * o Now provides a class-based iface to get to the incremental file info
 *
 * Revision 1.2  2004/10/10 06:07:46  me133
 * o Added header comment
 */

#include "StdAfx.h"

#include "WatcherSocket.h"	// Me

#include "Helpers.h"		// WaitForDispatchThreadEvents()

#include "IncrFile.h"		// CIncrBuildFile

#include <time.h>			// time()

static const TCHAR* szPORT_ENV_VAR_NAME = _T("EDIT_AND_CONTINUE_PORT");

using namespace std;

#define WSA_VERIFY_COND(cond, msg, severity)									\
	if (!(cond))																\
	{																			\
		return ILXLOG(HRESULT_FROM_WIN32(WSAGetLastError()), msg, severity);	\
	}

#define WSA_VERIFY_COND1(cond, severity, msg, arg1)									\
	if (!(cond))																	\
	{																				\
		return ILXLOG1(HRESULT_FROM_WIN32(WSAGetLastError()), severity, msg, arg1);	\
	}

#define WSA_VERIFY_COND_V(cond, msg, severity)									\
	if (!(cond))																\
	{																			\
		ILXLOG(HRESULT_FROM_WIN32(WSAGetLastError()), msg, severity);			\
		return;																	\
	}

#define WSA_VERIFY_COND_(ret, cond, msg, severity)								\
	if (!(cond))																\
	{																			\
		ILXLOG(HRESULT_FROM_WIN32(WSAGetLastError()), msg, severity);			\
		return ret;																\
	}

// Sync with EnCSocketServer.cs
enum NetworkMessage
{
	nTHUMP_THUMP		= 3,
	nMODULE_INFO		= 2,
	nCHANGED_MODULE		= 1,
	nSHUTDOWN			= 0, // This is sent by the same thread that sends MODULE_INFO
};

// HELPER FUNCTIONS
HRESULT blocking_recv(SOCKET s, char* buf, int len, int flags);

///////////////////////////////////
// PROFILER SOCKET IMPLEMENTATION
///////////////////////////////////

CWatcherSocket::CWatcherSocket()
	: m_socket(INVALID_SOCKET)
	, m_wsaEvent(0)
{ }

CWatcherSocket::~CWatcherSocket()
{
	if (m_wsaEvent)
	{
		BOOL bRet = WSACloseEvent(m_wsaEvent);
		WSA_VERIFY_COND_V(bRet, _T("WSACloseEvent() failed."), ILXLOG_ERROR);
	}

	WSACleanup();
}

HRESULT CWatcherSocket::Connect(int nPortToUse, int nRetries, bool bLogConnectFailure)
{
	HRESULT hr = S_OK;

	WSADATA wsaData;
	int nErr = WSAStartup( MAKEWORD( 2, 2 ), &wsaData );
	WSA_VERIFY_COND(nErr == 0, _T("WSAStartup() failed."), ILXLOG_ERROR);

	ILXLOGMSG1(ILXLOG_SUCCESS, _T("Connecting to Watcher at Port %d."), nPortToUse);

	sockaddr_in sa;
	sa.sin_family = AF_INET;
 	sa.sin_port = htons(nPortToUse);
	sa.sin_addr.S_un.S_addr = inet_addr( "127.0.0.1" );

	m_socket = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, 0, 0, 0);
	WSA_VERIFY_COND(m_socket != INVALID_SOCKET, _T("WSASocket() failed."), ILXLOG_ERROR);

	int nRetry = 0;
	static const int nTIME_TO_SLEEP_BETWEEN_RETRIES = 200; // ms

	int nResult = SOCKET_ERROR;
	int nLastError = 0;

	for(int nRetry = 0; nRetry < nRetries; ++nRetry)
	{
		nResult = connect(m_socket, (const sockaddr*) &sa, sizeof(sa));
		if (nResult == SOCKET_ERROR)
		{
			nLastError = ::WSAGetLastError();
		}
		else
		{
			ILXLOGMSG0(ILXLOG_SUCCESS, _T("Connected to Watcher."));
			return S_OK;
		}

		Sleep( nTIME_TO_SLEEP_BETWEEN_RETRIES );
	}

	hr = HRESULT_FROM_WIN32(nLastError);

	if (bLogConnectFailure)
	{
		ILXLOG1(hr, ILXLOG_ERROR, _T("connect() failed after %d retries."), nRetries);
	}

	// Couldn't connect the socket
	m_socket = INVALID_SOCKET;
	return hr;
}

bool CWatcherSocket::IsConnected() const
{
	return m_socket != INVALID_SOCKET;
}

HRESULT CWatcherSocket::SendModuleInfo(	DWORD dwModuleID,
										const char* szModulePath,
										const CCompileInfo* pCompileInfo)
{
	USES_CONVERSION;

	if (!pCompileInfo) 
		return E_POINTER;

	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	HRESULT hr = S_OK;

	ILXLOGMSG2(ILXLOG_INFO, _T("Sending MODULE INFO to Watcher: %d '%s'."),
		dwModuleID, szModulePath);

	hr = SendUINT32(nMODULE_INFO);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send network message id '%d'."), 
		nMODULE_INFO);

	// Send module id

	hr = SendUINT32(dwModuleID);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send module id %d."), dwModuleID);

	// Send module path

	hr = SendString(szModulePath);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send module path '%s'."), szModulePath);

	// Send compiler name

	hr = SendString(W2CA(pCompileInfo->getCompilerName()));
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send compiler name '%ls'."), 
		pCompileInfo->getCompilerName());

	// Send Debug flag

	hr = SendUINT32(pCompileInfo->getDebugFlag());
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send debug flag '%d'."), 
		pCompileInfo->getDebugFlag());

	// Send compiler options ("/unsafe /checked /o")

	hr = SendString(W2CA(pCompileInfo->getCompilerOptions()));
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send compiler info '% s'."), 
		pCompileInfo->getCompilerOptions());

	// Send source file names

	size_t nSourceFileCount = pCompileInfo->getSourceFileCount();

	hr = SendUINT32( (UINT32) nSourceFileCount);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send source file count %d."), 
		nSourceFileCount);

	for(size_t i = 0; i < nSourceFileCount; ++i)
	{
		hr = SendString(W2CA(pCompileInfo->getSourceFileName(i)));
		VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send string: '%ls'."), 
			pCompileInfo->getSourceFileName(i));
	}

	ILXLOGMSG0(ILXLOG_INFO, _T("Finished sending MODULE INFO to Watcher."));

	return S_OK;
}

WSAEVENT CWatcherSocket::GetReadEventHandle()
{
	if (m_socket == INVALID_SOCKET)
		return 0;

	HRESULT hr = S_OK;

	// Create the read event (lazy instantiation)

	if (!m_wsaEvent)
	{
		m_wsaEvent = WSACreateEvent();
		WSA_VERIFY_COND_(0, m_wsaEvent != WSA_INVALID_EVENT,
			_T("WSACreateEvent() failed."), ILXLOG_ERROR);

		// Associate the event with the read network event
		int nRet = WSAEventSelect(m_socket, m_wsaEvent, FD_READ);
		WSA_VERIFY_COND_(0, nRet == 0, _T("WSAEventSelect() failed."), ILXLOG_ERROR);
	}

	return m_wsaEvent;
}

HRESULT CWatcherSocket::RequestChangedModuleInfo()
{
	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	HRESULT hr = S_OK;

	hr = SendUINT32(nCHANGED_MODULE);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send network message id '%d'."), 
		nCHANGED_MODULE);

	return S_OK;
}

HRESULT CWatcherSocket::GetChangedModuleInfo(	DWORD* pdwModuleID,
												char* szModulePath,
												int nMaxLen,
												DWORD* pdwStartTicks,
												DWORD* pdwWeaveTimeEnd,
												DWORD* pdwCompileTimeEnd)
{
	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	if (!pdwModuleID || !szModulePath) return E_POINTER;
	if (nMaxLen <= 0) return E_INVALIDARG;

	HRESULT hr = S_OK;

	//hr = SendUINT32(CHANGED_MODULE);
	//VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send network message id '%d'."), 
	//	CHANGED_MODULE);

	// Turn off the event until we finish reading everything, otherwise we
	// could get a WSAEWOULDBLOCK from a recv() call.
	int nRet = WSAEventSelect(m_socket, m_wsaEvent, 0);
	WSA_VERIFY_COND(nRet == 0, _T("WSAEventSelect() failed."), ILXLOG_ERROR);

	// Set the socket I/O mode: In this case FIONBIO
	// enables or disables the blocking mode for the 
	// socket based on the numerical value of iMode.
	// If iMode = 0, blocking is enabled; 
	// If iMode != 0, non-blocking mode is enabled.
	u_long iMode = 0;
	nRet = ioctlsocket(m_socket, FIONBIO,  &iMode);
	if (nRet != 0)
	{
		hr = HRESULT_FROM_WIN32(WSAGetLastError());
		ILXLOG(hr, _T("ioctlsocket() failed."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	_ASSERTE(sizeof(DWORD) == sizeof(UINT32));

	//ILXLOGMSG0(ILXLOG_INFO, _T("Waiting for a new module."));

	hr = ReadUINT32( (UINT32*) pdwModuleID );
	if (FAILED(hr))
	{
		ILXLOG(hr, _T("Failed to retrieve module id from socket."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	//ILXLOGMSG1(ILXLOG_SUCCESS, _T("ReadUINT32() succeeded. Got Module ID: %d."), *pdwModuleID);

	hr = ReadString(szModulePath, nMaxLen);
	if (FAILED(hr))
	{
		ILXLOG(hr, _T("Failed to retrieve module path from socket."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	hr = ReadUINT32( (UINT32*) pdwStartTicks );
	if (FAILED(hr))
	{
		ILXLOG(hr, _T("Failed to retrieve start time from socket."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	hr = ReadUINT32( (UINT32*) pdwWeaveTimeEnd );
	if (FAILED(hr))
	{
		ILXLOG(hr, _T("Failed to retrieve weave end time from socket."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	hr = ReadUINT32( (UINT32*) pdwCompileTimeEnd );
	if (FAILED(hr))
	{
		ILXLOG(hr, _T("Failed to retrieve compile end time from socket."), ILXLOG_ERROR);
		goto EXIT_FUNC;
	}

	ILXLOGMSG5(ILXLOG_INFO, 
		_T("WATCHER THREAD: Got module update message ('%s', ModuleID: 0x%08X, Start:%d, Weave:%dms, Compile:%dms)."), 
		szModulePath,
		*pdwModuleID,
		*pdwStartTicks,
		*pdwWeaveTimeEnd,
		*pdwCompileTimeEnd);

EXIT_FUNC:
	// Turn the event back on while we wait for the next module change
	nRet = WSAEventSelect(m_socket, m_wsaEvent, FD_READ);
	WSA_VERIFY_COND(nRet == 0, _T("WSAEventSelect() failed."), ILXLOG_ERROR);

	ULONG ulDataAvailableCount = 0;
	nRet = ioctlsocket(m_socket, FIONREAD, &ulDataAvailableCount);
	WSA_VERIFY_COND(nRet == 0, _T("ioctlsocket(FIONREAD) failed."), ILXLOG_ERROR);
	if (ulDataAvailableCount > 0)
	{
		ILXLOGMSG1(ILXLOG_INFO, _T("The socket still has %d bytes remaining to be read."),
			ulDataAvailableCount);
	}
	else
	{
		WSAResetEvent(m_wsaEvent);
		_ASSERTE(WSAWaitForMultipleEvents(1, &m_wsaEvent, TRUE, 0, FALSE) 
			!= WSA_WAIT_EVENT_0);
	}

	return hr;
}

HRESULT CWatcherSocket::SendHeartbeat()
{
	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	HRESULT hr = S_OK;

	hr = SendUINT32(nTHUMP_THUMP);
	VERIFY_HR0(hr, ILXLOG_ERROR, _T("Failed to send heartbeat message."));

	UINT32 nHeartbeatResponse = 0;
	hr = ReadUINT32( (UINT32*) &nHeartbeatResponse );
	VERIFY_HR(hr, _T("Failed to receive heartbeat response from Watcher."), ILXLOG_ERROR);
	_ASSERTE(nHeartbeatResponse == nTHUMP_THUMP);

	return S_OK;
}

HRESULT CWatcherSocket::SendShutdown()
{
	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	HRESULT hr = S_OK;

	hr = SendUINT32(nSHUTDOWN);
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send shutdown message id '%d'."), 
		nSHUTDOWN);

	return S_OK;
}

HRESULT CWatcherSocket::SendUINT32(UINT32 ui)
{
	HRESULT hr = S_OK;

	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	int nSent = send(m_socket, (const char*) &ui, sizeof(ui), 0);
	WSA_VERIFY_COND(nSent == sizeof(ui), _T("send() failed."), ILXLOG_ERROR);

	return S_OK;
}

HRESULT CWatcherSocket::SendString(const char* sz, size_t nLen)
{
	HRESULT hr = S_OK;

	if (m_socket == INVALID_SOCKET)
		return E_FAIL;

	if (nLen == -1)
		nLen = strlen(sz);

	_ASSERTE(sizeof(UINT32) == sizeof(size_t));

	// Send the length first
	hr = SendUINT32( (UINT32) nLen );
	VERIFY_HR1(hr, ILXLOG_ERROR, _T("Failed to send string length: %d."), nLen);

	_ASSERTE(sizeof(int) == sizeof(size_t));

	// Send the string
	int nSent = send(m_socket, sz, (int) nLen, 0);
	WSA_VERIFY_COND(nSent == nLen, _T("send() failed."), ILXLOG_ERROR);

	return S_OK;
}

HRESULT CWatcherSocket::ReadUINT32(UINT32* pui)
{
	return blocking_recv(m_socket, (char*) pui, sizeof(UINT), 0);
}

HRESULT CWatcherSocket::ReadString(char* sz, size_t nMaxLen)
{
	HRESULT hr = S_OK;

	/*
	char sz2[50];
	blocking_recv(m_socket, sz2, 50, 0);
	for(int i = 0; i < 50; ++i)
	{
		if (i > 0 && i % 4 == 0) printf(" ");
		printf("%02X", sz2[i]);
	}
	printf("\n");
	*/

	char cLenOfString = 0;
	hr = blocking_recv(m_socket, &cLenOfString, sizeof(cLenOfString), 0);

	int nLenOfString = (int) cLenOfString;

	//ILXLOGMSG1(ILXLOG_SUCCESS, _T("ReadUINT32() success. Len=%d."), 
	//	(int) nLenOfString);

	if (nLenOfString > (int) nMaxLen)
	{
		return HRESULT_FROM_WIN32(ERROR_INSUFFICIENT_BUFFER);
	}

	return blocking_recv(m_socket, sz, nLenOfString, 0);
}

// HELPER FUNCTIONS

HRESULT blocking_recv(SOCKET s, char* buf, int len, int flags)
{
	HRESULT hr = S_OK;

	if (s == INVALID_SOCKET)
		return E_FAIL;

	u_long ulDataAvailableCount = 0;

	time_t timStart;
	time_t timCurrent;
	time(&timStart);

	static const int nTIMEOUT = 20; // secs
	static const int nSLEEP_TIME = 10; // ms

	while(time(&timCurrent) - timStart < nTIMEOUT)
	{
		int nRet = ioctlsocket(s, FIONREAD, &ulDataAvailableCount);
		WSA_VERIFY_COND(nRet == 0, _T("ioctlsocket(FIONREAD) failed."), ILXLOG_ERROR);

		if (ulDataAvailableCount >= (u_long) len)
		{
			break;
		}

		Sleep(nSLEEP_TIME);
	}

	if (ulDataAvailableCount < (u_long) len)
	{
		ILXLOGMSG1(ILXLOG_ERROR, _T("Gave up trying to read from the socket after %d seconds."),
			timCurrent - timStart);
		return E_FAIL;
	}

	int nRead = recv(s, buf, len, flags);
	WSA_VERIFY_COND1(nRead == len, ILXLOG_ERROR, _T("recv() failed. nRead=%d"), nRead);

	return S_OK;
}
