///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Watcher/AspectRules.cs 14    12/16/05 8:42p Eaddy $
//
//    File name:	AspectRules.cs
//   Created By:	Marc Eaddy
//   Created on:	10/17/04
//
//      Purpose:	
//
//  Description:	
//
// Currently, the C++/CLI and J# compilers do not work.  The C++/CLI CodeDomProvider
// returns null when CreateCompiler() is called.  The J# compiler compiles but there
// are problems when the function is patched that cause an InvalidProgramException.
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Watcher/AspectRules.cs $
 * 
 * 14    12/16/05 8:42p Eaddy
 * o Can now weave getters and setters
 * o Can now disable individual aspects
 * o Now longer watches disabled aspect source files
 * 
 * 13    12/15/05 2:40a Eaddy
 * o Fixed bug where we got exception if the aspect method was missing
 * o No longer tries to weave extern methods
 * 
 * 12    12/10/05 12:43a Eaddy
 * o Now keeps track of elapsed times
 * 
 * 11    12/08/05 3:56p Eaddy
 * o Fixed log message
 * 
 * 10    12/07/05 7:57p Eaddy
 * o Removed verbose logging
 * 
 * 9     12/07/05 12:41a Eaddy
 * o Improved AOP support
 * 
 * 8     5/28/05 1:24p Eaddy
 * o Changed copyright
 * 
 * 7     5/25/05 6:33p Eaddy
 * o Now uses EnC.Watcher namespace
 * 
 * 6     5/10/05 5:10p Eaddy
 * o Now fails gracefully for field decls
 * 
 * 5     4/22/05 7:36p Eaddy
 * o Can now turn off weaving using an attribute in the XML file
 * 
 * 4     4/15/05 2:49p Eaddy
 * o Added assert
 * 
 * 3     4/04/05 2:28p Eaddy
 * o Supports Properties
 * o Added AspectRulesFile class
 * 
 * 2     4/01/05 6:56p Eaddy
 * o Now handles parsing exceptions
 * 
 * 1     4/01/05 11:15a Eaddy
 * o Now supports rudimentary weaving
 */

using System;
using System.IO;
using System.Reflection;
using System.Diagnostics;
using System.Collections;
using System.Xml;
using System.Xml.Serialization;

using DDW.CSharp.Dom;

namespace EnC.Watcher
{
	public enum JPType
	{
		uninitialized = -1,
		call,
		execution
	}

	public enum AdviceType
	{
		uninitialized = -1,
		before,
		after,
		around
	}

	public class IGraphMatch
	{
		public IGraphMatch(IGraph parent, IGraph match)
		{
			this.match = match;
			this.parent = parent;
		}

		public IGraph match;
		public IGraph parent;
	}

	public class Pointcut
	{
		[XmlAttribute("name")]
		public string name;

		[XmlAttribute("expression")]
		public string expression;

		public JPType jpType = JPType.uninitialized;

		public string signaturePattern;

		public ArrayList codeElementGraphTypes = null;

		public void init()
		{
			if (expression == null || expression.Length == 0)
			{
				MyTrace.Log(TraceLevel.Error, "Pointcut expression attribute cannot be empty.", 
					expression);
				return;
			}

			int posFirstParen = expression.IndexOf('(');
			if (posFirstParen < 0)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid pointcut expression '{0}'. Missing parenthesis.", 
					expression);
				return;
			}

			int posLastParen = expression.LastIndexOf(')');
			if (posLastParen < 0)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid pointcut expression '{0}'. Missing parenthesis.", 
					expression);
				return;
			}
			else if (posLastParen < posFirstParen)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid pointcut expression '{0}'. Improper parentheses.", 
					expression);
				return;
			}

			string jpTypeStr = expression.Substring(0, posFirstParen);

			try
			{
				jpType = (JPType) Enum.Parse(typeof(JPType), jpTypeStr, true);
			}
			catch (System.ArgumentException ex)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to parse pointcut expression '{0}': {1}", 
					expression,
					ex.Message);
				return;
			}

			signaturePattern = expression.Substring(posFirstParen+1, posLastParen-posFirstParen-1);
			if (signaturePattern == null || signaturePattern.Length == 0)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid signature in pointcut expression '{0}': Signature is empty or null.", 
					expression);
				return;
			}

			if (codeElementGraphTypes == null)
				codeElementGraphTypes = new ArrayList();

			switch(jpType)
			{
				case JPType.call:
				case JPType.execution:
				
					object[] objs = {	new DDW.CSharp.Dom.MethodDecl(), 
									   new DDW.CSharp.Dom.AccessorDecl() };

					foreach(object obj in objs)
					{
						IDeclaration decl = obj as IDeclaration;
						if (decl != null)
						{
							codeElementGraphTypes.Add(decl.GraphType);
							continue;
						}

						IScope decl2 = obj as IScope;
						if (decl2 != null)
						{
							codeElementGraphTypes.Add(decl2.GraphType);
							continue;
						}
					}

					break;
				default:
					MyTrace.Log(TraceLevel.Error, "Invalid joinpoint type '{0}'.", 
						jpType);
					return;
			}
		}
	}

	public class Advice
	{
		[XmlAttribute("name")]
		public string methodName;

		[XmlAttribute("type")]
		public string adviceTypeString;

		[XmlElement("pointcut")]
		public Pointcut[] pointcuts = null;

		public AdviceType adviceType;

		MethodDecl adviceMethod = null;

		static Type typeCEA = typeof(DDW.CSharp.Dom.CodeElementAttribute);

		public void init(string className, IGraph ig)
		{
			if (methodName == null || methodName.Length == 0)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid advice. Name attribute is missing or empty.");
				return;
			}

			if (adviceTypeString == null || adviceTypeString.Length == 0)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid advice for '{0}'. Advice type attribute ('type') is missing or empty.",
					methodName);
				return;
			}

			try
			{
				adviceType = (AdviceType) Enum.Parse(typeof(AdviceType), adviceTypeString);
			}
			catch (System.ArgumentException ex)
			{
				MyTrace.Log(TraceLevel.Error, "Invalid advice type '{0}': {1}", 
					adviceTypeString,
					ex.Message);
				return;
			}

			foreach(Pointcut pointcut in pointcuts)
			{
				pointcut.init();
			}

			adviceMethod = findMethod(ig, className + "." + methodName);
			if (adviceMethod == null)
			{
				MyTrace.Log(TraceLevel.Error, "Advice method '{0}.{1}' not found.", 
					className,
					methodName);
				return;
			}

			ArrayList matches = new ArrayList();

			findElements(null, adviceMethod, GraphTypes.UnknownReference, "_WICCA_*", matches);

			for(int i = 0; i < matches.Count; ++i)
			{
				UnknownReference unkRef = ((IGraphMatch) matches[i]).match as UnknownReference;
				Debug.Assert(unkRef != null);
				Debug.Assert(unkRef.Definition != null);
				unkRef.Definition.Name = unkRef.Definition.Name.Replace("_WICCA_", "");
			}
		}

		public int instrument(IGraph ig)
		{
			Debug.Assert(pointcuts != null);

			ArrayList matches = new ArrayList();

			foreach(Pointcut pointcut in pointcuts)
			{
				Debug.Assert(pointcut != null);
				string pattern = pointcut.signaturePattern;
				pattern = pattern.Replace("()", "");

				findElements(null, ig, pointcut.codeElementGraphTypes, 
					pattern, matches);
			}

			// Keeps track of the number of instrumented code elements
			int countInstrumented = 0;

			for(int i = 0; i < matches.Count; ++i)
			{
				IGraph igMatch = ((IGraphMatch) matches[i]).match;

				bool instrBefore = instrumentBefore(igMatch);
				bool instrAfter  = instrumentAfter(igMatch);
				bool instrAround = instrumentAround(igMatch);

				if (instrBefore || instrAfter || instrAround)
					++countInstrumented;
			}

			return countInstrumented;
		}

		public bool instrumentBefore(IGraph ig)
		{
			if (adviceType != AdviceType.before)
				return false;

			if (ig.GraphType == GraphTypes.MethodDecl)
			{
				MethodDecl declMethod = (MethodDecl) ig;

				// Skip extern methods since they don't have a body
				if ((declMethod.Attributes & Modifiers.Extern) != 0)
					return false;

				//System.Console.WriteLine("Method: {0} Attributes: {1}",
				//	declMethod.Name, declMethod.Attributes);

				return instrumentBefore(declMethod.Statements);
			}
			else if (ig.GraphType == GraphTypes.AccessorDecl)
			{
				AccessorDecl declAccessor = (AccessorDecl) ig;
				if ((declAccessor.Attributes & Modifiers.Abstract) != 0)
					return false;

				return instrumentBefore(declAccessor.Statements);
			}
			else if (ig.GraphType == GraphTypes.PropertyDecl)
			{
				PropertyDecl declProp = (PropertyDecl) ig;
				instrumentBefore(declProp.GetAccessor.Statements);
				return instrumentBefore(declProp.SetAccessor.Statements);
			}

			return false;
		}

		private bool instrumentBefore(StatementCollection statements)
		{
			if (adviceType != AdviceType.before)
				return false;

			for(int i = 0; i < adviceMethod.Statements.Count; ++i)
			{
				statements.Insert(i, adviceMethod.Statements[i]);
			}

			return adviceMethod.Statements.Count > 0;
		}

		public bool instrumentAfter(IGraph ig)
		{
			if (adviceType != AdviceType.after)
				return false;

			if (ig.GraphType == GraphTypes.MethodDecl)
			{
				MethodDecl declMethod = (MethodDecl) ig;

				// Skip extern methods since they don't have a body
				if ((declMethod.Attributes & Modifiers.Extern) != 0)
					return false;

				return instrumentAfter(declMethod.Statements);
			}
			else if (ig.GraphType == GraphTypes.AccessorDecl)
			{
				AccessorDecl declAccessor = (AccessorDecl) ig;
				if ((declAccessor.Attributes & Modifiers.Abstract) != 0)
					return false;

				return instrumentAfter(declAccessor.Statements);
			}
			else if (ig.GraphType == GraphTypes.PropertyDecl)
			{
				PropertyDecl declProp = (PropertyDecl) ig;

				instrumentAfter(declProp.GetAccessor.Statements);
				return instrumentAfter(declProp.SetAccessor.Statements);
			}

			return false;
		}

		private bool instrumentAfter(StatementCollection body)
		{
			if (adviceType != AdviceType.after)
				return false;

			//const bool useTryFinally = false;

			//if (useTryFinally)
			//{
			//	StatementCollection bodyCopy = new StatementCollection(body);
			//	TryCatchFinallyStmt tryCatchFinallyStmt = new TryCatchFinallyStmt();
			//	tryCatchFinallyStmt.Try.AddRange(bodyCopy);
			//	tryCatchFinallyStmt.Finally.AddRange(adviceMethod.Statements);
			//	body.Clear();
			//	body.Add(tryCatchFinallyStmt);
			//}
			//else
			{
				// Search for all the return statements

				ArrayList matches = new ArrayList();
				findElements(null, body, GraphTypes.ReturnStmt, matches);

				for(int i = 0; i < matches.Count; ++i)
				{
					IGraphMatch matchReturn = (IGraphMatch) matches[i];
			
					StatementCollection statements = matchReturn.parent as StatementCollection;
					if (statements == null)
						continue; // We only know how to instrument statement collections

					// Locate the matched return statement. The remaining return statements
					// will be instrumented in subsequent iterations. This handles multiple return
					// statements in the same StatementCollection as well as return statements in
					// other StatementCollections.
					int indexRetStmt = statements.IndexOf( (Statement) matchReturn.match);
					Debug.Assert(indexRetStmt >= 0);

					// Insert the after aspect statements before the return statement
					for(int k = 0; k < adviceMethod.Statements.Count; ++k)
					{
						statements.Insert(indexRetStmt+k, adviceMethod.Statements[k]);
					}
				}

				int count = body.Count;
				if (count > 0 && body[count-1].GraphType != GraphTypes.ReturnStmt)
				{
					body.AddRange(adviceMethod.Statements);
				}
			}

			return true;
		}

		public bool instrumentAround(IGraph ig)
		{
			if (adviceType != AdviceType.around)
				return false;

			if (ig.GraphType == GraphTypes.MethodDecl)
			{
				MethodDecl declMethod = (MethodDecl) ig;

				// Skip extern methods since they don't have a body
				if ((declMethod.Attributes & Modifiers.Extern) != 0)
					return false;

				declMethod.Statements.Clear();
				declMethod.Statements.AddRange(adviceMethod.Statements);

				return true;
			}
			else if (ig.GraphType == GraphTypes.AccessorDecl)
			{
				AccessorDecl declAccessor = (AccessorDecl) ig;
				if ((declAccessor.Attributes & Modifiers.Abstract) != 0)
					return false;

				declAccessor.Statements.Clear();
				declAccessor.Statements.AddRange(adviceMethod.Statements);

				return true;
			}
			else if (ig.GraphType == GraphTypes.PropertyDecl)
			{
				PropertyDecl declProp = (PropertyDecl) ig;

				declProp.GetAccessor.Statements.Clear();
				declProp.GetAccessor.Statements.AddRange(adviceMethod.Statements);
				declProp.SetAccessor.Statements.Clear();
				declProp.SetAccessor.Statements.AddRange(adviceMethod.Statements);

				return true;
			}

			return false;
		}

		#if DEBUG
		static int depth = -1;
		#endif

		public static string getFriendlyString(IGraph element)
		{
			string name = Helper.ReflectGetProperty(element, "Name");
			if (name != null && name.Length > 0)
			{
				return name;
			}

			UnknownReference unkRef = element as UnknownReference;
			if (unkRef != null)
			{
				name = unkRef.VariableName;
			}

			if (name != null && name.Length > 0)
			{
				return name;
			}

			MethodRef methodRef = element as MethodRef;
			if (methodRef != null)
			{
				name = methodRef.MethodName;
			}

			if (name != null && name.Length > 0)
			{
				return name;
			}

//			MemberAccess memberAccess = element as MemberAccess;
//			if (memberAccess != null)
//			{
//				name = getFriendlyString(memberAccess.Target) + "." + memberAccess.MemberName;
//			}
//
//			if (name != null && name.Length > 0)
//			{
//				return name;
//			}

			return "";
		}

		public static int findElements(IGraph parent, IGraph element, 
			GraphTypes elemType, ArrayList matches)
		{
			System.Text.RegularExpressions.Regex pattern = null;

			return findElements(parent, element, elemType, pattern, matches);
		}

		public static int findElements(IGraph parent, IGraph element, 
			ArrayList elemTypes, string pattern, ArrayList matches)
		{
			if (elemTypes == null)
				return 0;

			System.Text.RegularExpressions.Regex regexPattern =
				new System.Text.RegularExpressions.Regex(pattern);

			int total = 0;

			foreach(GraphTypes elemType in elemTypes)
			{
				int count = findElements(parent, element, elemType, regexPattern, matches);
				total += count;
			}

			return total;
		}

		public static int findElements(IGraph parent, IGraph element, 
			GraphTypes elemType, string pattern, ArrayList matches)
		{
			return findElements(parent, element, elemType,
				new System.Text.RegularExpressions.Regex(pattern), matches);
		}

		public static int findElements(IGraph parent, IGraph element, 
			GraphTypes elemType, System.Text.RegularExpressions.Regex pattern, ArrayList matches)
		{
			#if DEBUG
			++depth;
			#endif

			#if false
			for(int i = 0; i < depth; ++i)
			{
				System.Console.Write(" ");
			}

			System.Console.WriteLine("{0} ({1})", 
				getFriendlyString(element),
				element.GraphType);
			#endif

			Debug.Assert(element != null);

			if (elemType == (GraphTypes)(-1) || element.GraphType == elemType)
			{
				if (pattern == null)
				{
					matches.Add(new IGraphMatch(parent, element));
				}
				else if (elemType == GraphTypes.UnknownReference)
				{
					UnknownReference unkRef = element as UnknownReference;
					Debug.Assert(unkRef != null);

					if (pattern.Match(unkRef.VariableName).Success)
					{
						matches.Add(new IGraphMatch(parent, element));
					}
				}
				else if (elemType == GraphTypes.MethodDecl)
				{
					string parentName = "";

					MethodDecl methodDecl = element as MethodDecl;
					Debug.Assert(methodDecl != null);

					DDW.CSharp.SymbolTable.TypeScope typeScope = 
						methodDecl.Scope.EnclosingScope as DDW.CSharp.SymbolTable.TypeScope;
					Debug.Assert(typeScope != null);

					if (typeScope != null)
					{
						parentName = Helper.ReflectGetProperty(typeScope.Definition, "Name");
						if (parentName != null && parentName.Length > 0)
						{
							parentName += ".";
						}
					}

					string elementName = Helper.ReflectGetProperty(element, "Name");

					string fullName = parentName + elementName;

					#if false
					for(int i = 0; i < depth; ++i)
					{
						System.Console.Write(" ");
					}
					System.Console.WriteLine("{0}=={1}?", fullName, pattern);
					#endif

					if (pattern.Match(fullName).Success)
					{
						matches.Add(new IGraphMatch(parent, element));
					}
				}
				else if (elemType == GraphTypes.AccessorDecl)
				{
					AccessorDecl accessorDecl = element as AccessorDecl;
					Debug.Assert(accessorDecl != null);

					Debug.Assert(parent != null);
					PropertyDecl propertyDecl = parent as PropertyDecl;
					if (propertyDecl != null && propertyDecl.Definition != null)
					{
						Debug.Assert(propertyDecl != null);
						Debug.Assert(propertyDecl.Definition != null);
						Debug.Assert(propertyDecl.Definition.Scope != null);

						DDW.CSharp.SymbolTable.TypeScope typeScope = 
							propertyDecl.Definition.Scope as DDW.CSharp.SymbolTable.TypeScope;
						Debug.Assert(typeScope != null);

						string fullName = Helper.ReflectGetProperty(typeScope.Definition, "Name");
						Debug.Assert(fullName != null && fullName.Length > 0);

						if (accessorDecl.AccessorModifier != DDW.CSharp.Dom.AccessorModifiers.Empty)
						{
							fullName += "." + accessorDecl.AccessorModifier.ToString().ToLower() + "_";
						}
						else
						{
							fullName += ".";
						}

						string elementName = Helper.ReflectGetProperty(parent, "Name");
						Debug.Assert(elementName != null && elementName.Length > 0);

						fullName += elementName;

						MyTrace.Log(TraceLevel.Info, "{0}=={1}?", fullName, pattern);

						if (pattern.Match(fullName).Success)
						{
							matches.Add(new IGraphMatch(parent, element));
						}
					}
				}
			}

			if (element is ICollection)
			{
				ICollection children = (ICollection) element;
				foreach(IGraph child in children)
				{
					if (child != null)
						findElements(element, child, elemType, pattern, matches);
				}
			}

			Type type = element.GetType();

			MemberInfo[] props = type.FindMembers(MemberTypes.Property,
				BindingFlags.Public | BindingFlags.Instance,
				new MemberFilter(MemberFilter),
				null);

			foreach(MemberInfo prop in props)
			{
				object codeElementProp = type.InvokeMember(	prop.Name, 
					BindingFlags.Public | BindingFlags.GetProperty |  BindingFlags.Instance, 
					null, 
					element, 
					new object[]{} );

				if (codeElementProp is ICollection)
				{
					ICollection children = (ICollection) codeElementProp;
					parent = (IGraph) codeElementProp;
					foreach(IGraph child in children)
					{
						if (child != null)
							findElements(parent, child, elemType, pattern, matches);
					}
				}
				else if (codeElementProp is IGraph)
				{
					findElements(element, (IGraph) codeElementProp, elemType, pattern, matches);
				}
			}

			#if DEBUG
			--depth;
			#endif

			return matches.Count;
		}


		public static MethodDecl findMethod(IGraph element, string methodName)
		{
			ArrayList matches = new ArrayList();
			findElements(null, element, GraphTypes.MethodDecl, methodName, matches);
			if (matches.Count > 0)
			{
				// Just grab the first one
				return (MethodDecl) ((IGraphMatch) matches[0]).match;
			}
			else
			{
				return null;
			}
		}

		public static bool MemberFilter(MemberInfo objMemberInfo, Object objSearch)
		{			
			if(objMemberInfo.Name == "LinePragma") 
				return false;
			
			return objMemberInfo.IsDefined(typeCEA, true);
		}
	}

	public class Aspect
	{
		[XmlAttribute("namespace")]
		public string namespaceName;

		[XmlAttribute("class")]
		public string className;

		[XmlAttribute("sources")]
		public string sources;

		[XmlAttribute("assembly")]
		public string assembly;

		[XmlAttribute("enabled")]
		public bool enabled = true;

		[XmlElement("pointcut", typeof(Pointcut))]
		[XmlElement("advice", typeof(Advice))]
		public object[] namedPointcutsAndAdvice = null;

		IGraph ig = null;

		public void init()
		{
			Debug.Assert(sources != null);

			try
			{
				Helper helper = new Helper();

				ig = helper.GetGraph(sources);
				if (ig == null)
				{
					return;
				}
			}
			catch(Exception ex)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to parse source file '{0}': {1}",
					sources, ex.ToString());
				return;
			}

			foreach(object pcOrAdvice in namedPointcutsAndAdvice)
			{
				Pointcut pc = pcOrAdvice as Pointcut;
				if (pc != null)
				{
					pc.init();
				}
				else
				{
					Advice advice = pcOrAdvice as Advice;
					if (advice != null)
					{
						advice.init(className, ig);
					}
				}
			}
		}

		public void instrument(IGraph ig, ArrayList aspectAssemblies)
		{
			foreach(object pcOrAdvice in namedPointcutsAndAdvice)
			{
				Advice advice = pcOrAdvice as Advice;
				if (advice != null)
				{
					int count = advice.instrument(ig);
					if (count > 0 && assembly != null)
					{
						aspectAssemblies.Add(assembly);
					}
				}
			}
		}
	}

	[XmlRoot("aspects")]
	public class Aspects
	{
		[XmlAttribute("enable")]
		public bool enableWeaving = true;

		[XmlElement("aspect")]
		public Aspect[] rules = null;

		static public Aspects load(string path)
		{
			try
			{
				TextReader reader =	new	StreamReader(path);
				XmlSerializer serializer = new XmlSerializer(typeof(Aspects));
				Aspects aspects = (Aspects) serializer.Deserialize(reader);
				reader.Close();
				return aspects;
			}
			catch(XmlException ex)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to parse XML Aspects file: {0}.",
					ex.Message);
				return null;
			}
			catch(InvalidOperationException	ex)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to parse XML Aspects file: {0}.",
					ex.InnerException.Message);
				return null;
			}
			catch(Exception ex)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to parse XML Aspects file: {0}",
					ex.Message);
				return null;
			}
		}

		public void init()
		{
			foreach(Aspect rule in rules)
			{
				if (rule.enabled)
				{
					rule.init();
					MyTrace.Log(TraceLevel.Info, "Aspect rule enabled: {0}",
						rule.className);
				}
				else
				{
					MyTrace.Log(TraceLevel.Info, "Aspect rule disabled: {0}",
						rule.className);
				}
			}
		}

		public void instrument(IGraph ig, ArrayList aspectAssemblies)
		{
			if (ig == null)
				return;

			foreach(Aspect rule in rules)
			{
				if (rule.enabled)
					rule.instrument(ig, aspectAssemblies);
			}
		}
	}

	public class AspectsFile : FileWatcherCollection
	{
		/// <summary>
		/// Delegate that we call when an aspect file has been changed.
		/// </summary>
		event MyFileWatcher.OnChangedDelegate _onFileChangedEvent_ForClients;

		/// <summary>
		/// Delegate that MyFileWatcher calls when a file has changed. We cache this for efficiency.
		/// </summary>
		readonly MyFileWatcher.OnChangedDelegate _onFileChangedDelegate_ForMyself;

		/// <summary>
		/// Aspects
		/// </summary>
		Aspects				_aspects = null;

		/// <summary>
		/// Aspect rule filename.
		/// </summary>
		const string			_aspectFile = "AspectRules.xml";

		public AspectsFile()
		{
			// Initialize the file watcher changed delegate
			_onFileChangedDelegate_ForMyself = new MyFileWatcher.OnChangedDelegate(OnFileChanged);

			// Load the aspect rules
			loadAspects(_aspectFile);

			// Watch the aspect rule file for changes
			if (_aspects != null)
				registerForChanges(_aspectFile);
		}

		/// <summary>
		/// Loads the aspect rule file.
		/// </summary>
		/// <param name="pathAspects">Path to aspect rule file</param>
		private void loadAspects(string pathAspects)
		{
			// Step 1:	Stop watching the old aspect source files

			if (_aspects != null)
			{
				foreach(Aspect rule in _aspects.rules)
				{
					if (rule.enabled)
						unregisterForChanges(rule.sources);
				}
			}

			// Step 2:	Load the new aspect rules

			_aspects = Aspects.load(pathAspects);
			if (_aspects == null)
			{
				MyTrace.Log(TraceLevel.Error, "Failed to load aspect rules.");
				return;
			}

			// Step 3:	Initialize aspect rules
			//
			//			This causes the aspect source files to be parsed.


			if (!Enabled)
			{
				MyTrace.Log(TraceLevel.Info, "Aspects are disabled");
				return;
			}

			_aspects.init();

			// Step 4:	Watch for changes to the aspect source files

			foreach(Aspect rule in _aspects.rules)
			{
				Debug.Assert(rule.sources != null);
				if (rule.enabled)
					registerForChanges(rule.sources);
			}
		}

		/// <summary>
		/// <seealso cref="Aspects.instrument"/>
		/// </summary>
		/// <param name="ig"></param>
		public void instrument(IGraph ig, ArrayList aspectAssemblies)
		{
			if (_aspects != null)
			{
				lock(_aspects)
				{
					_aspects.instrument(ig, aspectAssemblies);
				}
			}
		}

		/// <summary>
		/// Sets the callback used to notify when a module has been recompiled.
		/// </summary>
		/// <param name="onFileChangedDelegate_ForClients"></param>
		public void setFileChangedDelegate(MyFileWatcher.OnChangedDelegate onFileChangedDelegate_ForClients)	
		{
			lock(this)
			{
				_onFileChangedEvent_ForClients += onFileChangedDelegate_ForClients;
			}
		}

		/// <summary>
		/// Internal callback called by MyFileWatcher when a file is changed.
		/// </summary>
		/// <param name="path">Path of changed file.</param>
		protected override void OnFileChanged(string path)
		{
			MyTrace.Log(TraceLevel.Info, "Change detected: " + path + ". Reloading aspects.");

			// If the aspect rule file changed or one of the aspect source files then
			// re-load the aspect rules and re-parse the source files.
			lock(_aspectFile)
			{
				loadAspects(_aspectFile);
			}

			// Notify interested listeners

			if (_onFileChangedEvent_ForClients != null)
			{
				lock(_onFileChangedEvent_ForClients)
				{
					_onFileChangedEvent_ForClients(path);
				}
			}
		}

		public bool Enabled
		{
			get { return _aspects != null && _aspects.enableWeaving; }
		}
	}
}
