///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Watcher/EnCSocketServer.cs 8     12/15/05 2:41a Eaddy $
//
//    File name:	EnCSocketServer.cs
//   Created By:	Marc Eaddy
//   Created on:	10/10/04
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Watcher/EnCSocketServer.cs $
 * 
 * 8     12/15/05 2:41a Eaddy
 * o Fixed stats
 * 
 * 7     12/10/05 12:43a Eaddy
 * o Now keeps track of elapsed times
 * 
 * 6     11/11/05 5:37p Eaddy
 * o Now displays more bytes
 * 
 * 5     5/30/05 6:30p Eaddy
 * o Fixed spelling mistake
 * 
 * 4     5/28/05 1:23p Eaddy
 * o Changed copyright
 * 
 * 3     5/25/05 6:34p Eaddy
 * o Now uses EnC.Watcher namespace
 * 
 * 2     3/10/05 12:42p Eaddy
 * o Can now handle long compiler options string
 * Revision 1.9  2005/01/14 23:32:35  me133
 * o Imiproved logging
 *
 * Revision 1.8  2004/10/26 04:46:05  me133
 * o Doesn't check for a heartbeat if PATCHER_BREAK is true since this
 * probably means the heartbeat would be paused
 *
 * Revision 1.7  2004/10/24 19:31:03  me133
 * o Implemented heartbeat message
 * o Now handles more network error conditions
 *
 * Revision 1.6  2004/10/17 08:45:25  me133
 * o Now uses DynModule which keeps track of all the compiler settings
 * and references (it gets these from Patcher)
 *
 * Revision 1.5  2004/10/12 04:02:42  me133
 * o No longer logs the stack trace twice
 *
 * Revision 1.4  2004/10/11 05:57:53  me133
 * no message
 *
 * Revision 1.3  2004/10/10 07:40:07  me133
 * o Now outputs trace messages to log file usnig System.Diagnostics.Trace
 *
 * Revision 1.2  2004/10/10 06:07:24  me133
 * o Added header comment
 */

using System;
using System.IO;
using System.Threading;
using System.Net;
using System.Net.Sockets;
using System.Diagnostics;

struct ModuleAndPath
{
	public uint moduleID;
	public string modulePath;
	public int startTime;
	public int weaveTimeEnd;
	public int compileTimeEnd;
};

namespace EnC.Watcher
{
	class Globals
	{
		// Sync with ProfilerSocket.cpp
		public enum NetworkMessage
		{
			nTHUMP_THUMP		= 3,
			nMODULE_INFO		= 2,
			nCHANGED_MODULE		= 1,
			nSHUTDOWN			= 0,	// This is not really a network message, its a 
			// module id that means its time to shutdown.
		};

		//public const uint WSAECONNRESET				= 10054;

		public static int nINT_SIZE = System.Runtime.InteropServices.Marshal.SizeOf(nINT_SIZE);

		static public string ReadLengthEncodedASCIIString( BinaryReader br )
		{
			uint nLength = br.ReadUInt32();
			if ( nLength > 20000 || nLength < 0 )
			{
				byte[] abNextBytes = new byte[ 500 ];
				br.Read( abNextBytes, 0, 8 );
				string strError = "Length was abnormally large or small (" + nLength.ToString( "x" ) + ").  Next bytes were ";
				foreach ( byte b in abNextBytes )
					strError += b.ToString( "x" ) + " (" + ( Char.IsControl( ( char )b ) ? '-' : ( char )b ) + ") ";

				throw new InvalidOperationException( strError );
			}

			byte[] abString = new byte[ nLength ];
			int nRead = 0;
				
			DateTime dt = DateTime.Now;

			while ( nRead < nLength )
			{
				nRead += br.Read( abString, nRead, (int) nLength - nRead );

				// Make this loop finite (30 seconds)
				TimeSpan ts = DateTime.Now - dt;
				if ( ts.TotalSeconds > 30 )
					throw new InvalidOperationException( "Timed out while waiting for length encoded string" );
			}

			return System.Text.ASCIIEncoding.ASCII.GetString( abString, 0, (int) nLength );
		}

		static public uint ReadUINT32(Socket socket)
		{
			byte[] buf = new byte[Globals.nINT_SIZE];

			int nRead = socket.Receive(buf); // This could hang indefinitely
			if (nRead == 0)
			{
				//MyTrace.Log(TraceLevel.Info, 
				//	"ModuleReceiverWorker thread stopping because _socket is closed");
				return 0;
			}

			Debug.Assert(	nRead == Globals.nINT_SIZE && 
				buf.Length == Globals.nINT_SIZE);

			return BitConverter.ToUInt32(buf, 0);
		}
	}

	/// <summary>
	/// Summary description for EnCSocketServer.
	/// </summary>
	public class EnCSocketServer
	{
		// DATA

		private ModuleReceiverWorker	_moduleReceiverWorker	= new ModuleReceiverWorker();
		private ModuleSenderWorker		_moduleSenderWorker		= new ModuleSenderWorker();
		private HeartbeatWorker			_heartbeatWorker		= new HeartbeatWorker();
		private ManualResetEvent		_mreShutdownSignaled	= new ManualResetEvent(false);
		private ModuleDelegate			_delegateOnGotModuleInfo;
		private int						_nPort;
		private bool					_bUseHeartbeat;

		// CTOR

		public EnCSocketServer(int nPort, bool bUseHeartbeat, ModuleDelegate onGotModuleInfo)
		{
			_nPort = nPort;
			_bUseHeartbeat = bUseHeartbeat;

			if (_nPort != 0)
			{
				MyTrace.Log(TraceLevel.Info, "Watcher server is using port {0}.", _nPort);
			}

			_delegateOnGotModuleInfo = onGotModuleInfo;
		}

		// METHODS

		public void StartListening()
		{
			try
			{
				TcpListener tcpListener = new TcpListener(IPAddress.Loopback, _nPort);

				// Start listening for connections
				tcpListener.Start();

				// Keep listening until we have two connections
				while (	!_moduleReceiverWorker.IsStarted() || 
					!_moduleSenderWorker.IsStarted() ||
					!_heartbeatWorker.IsStarted())
				{
					MyTrace.Log(TraceLevel.Info, "Waiting for a connection...");
	        
					// Perform a blocking call to accept requests.
					// Ideally, we would give up after a few seconds.
					Socket socket = tcpListener.AcceptSocket();

					MyTrace.Log(TraceLevel.Info, "Connected!");

					// Get the Message Type

					Globals.NetworkMessage networkMsg = 
						(Globals.NetworkMessage) Globals.ReadUINT32(socket);

					MyTrace.Log(TraceLevel.Info, "GOT CLIENT TYPE={0}", networkMsg.ToString());

					// Spawn the appropriate worker thread class

					if (networkMsg == Globals.NetworkMessage.nMODULE_INFO)
					{
						_moduleReceiverWorker.StartProcessing(	socket, 
							_delegateOnGotModuleInfo);
					}
					else if (networkMsg == Globals.NetworkMessage.nTHUMP_THUMP)
					{
						if (_bUseHeartbeat)
						{
							_heartbeatWorker.StartProcessing2(	socket,
								new ShutdownDelegate(StopAndWaitCallback));
						}
						else
						{
							MyTrace.Log(TraceLevel.Info, 
								"Ignoring heartbeat message because Patcher is running in the debugger (PATCHER_BREAK=1).");
						}
					}
					else if (networkMsg == Globals.NetworkMessage.nCHANGED_MODULE)
					{
						_moduleSenderWorker.StartProcessing(socket);
					}
					else
					{
						Debug.Assert(false, "Can't happen - NetworkMessage: " + networkMsg.ToString());
					}
				}

				// We've got the two connections we were expecting, stop listening
				tcpListener.Stop();
				tcpListener = null; // So we don't accidentally use
			}
			catch (SocketException ex)
			{
				if (ex.Message.IndexOf("forcibly") > -1)
				{
					// Patcher died (CTRL^C)
					MyTrace.Log(TraceLevel.Info, 
						"EncSocketServer - Patcher has left the building. Exiting...");
				}
				else
				{
					MyTrace.Log(TraceLevel.Error, "socket error: " + ex.ToString());
				}
			}
			catch (Exception ex)
			{
				MyTrace.Log(TraceLevel.Error, "socket error: " + ex.ToString());
			}

			// FIX ME (REMOVE)
			//Console.WriteLine("Ready for patching");
		}

		/// <summary>
		/// This should only be called by CModuleReceiverWorker
		/// </summary>
		private void StopAndWaitCallback()
		{
			MyTrace.Log(TraceLevel.Info, "StopAndWaitCallback called. Shutting down.");
			StopAndWait();
		}

		/// <summary>
		/// This is only called when we did not spawn the target process. Instead,
		/// the Profiler has spawned us.  In this case, we should handle network
		/// communication until the Profiler eventually sends us a shutdown message. 
		/// </summary>
		public void WaitForShutdown()
		{
			_mreShutdownSignaled.WaitOne();
		}

		/// <summary>
		/// Shuts down everything. Call this if we are exiting.
		/// </summary>
		public void StopAndWait()
		{
			MyTrace.Log(TraceLevel.Info, "StopAndWait called. Shutting down.");

			_mreShutdownSignaled.Set();

			if (_moduleReceiverWorker == null && 
				_moduleSenderWorker == null &&
				_heartbeatWorker == null)
			{
				return;
			}

			_moduleReceiverWorker.RequestStop();
			_moduleSenderWorker.RequestStop();
			_heartbeatWorker.RequestStop();

			MyTrace.Log(TraceLevel.Info, "Waiting for threads to die");

			const int nTimeoutSecs = 5; // secs
			TimeSpan timeout = new TimeSpan(0, 0, nTimeoutSecs);

			_moduleReceiverWorker.WaitForThreadToDie(timeout);
			MyTrace.Log(TraceLevel.Info, "_moduleReceiverWorker dead");

			_moduleSenderWorker.WaitForThreadToDie(timeout);
			MyTrace.Log(TraceLevel.Info, "_moduleSenderWorker dead");

			_heartbeatWorker.WaitForThreadToDie(timeout);
			MyTrace.Log(TraceLevel.Info, "_heartbeatWorker dead");

			_moduleReceiverWorker = null;
			_moduleSenderWorker = null;
			_heartbeatWorker = null;

			MyTrace.Log(TraceLevel.Info, "Threads dead");
		}

		/// <summary>
		/// This is called by DynClass when a module has been updated. We need to
		/// send the module info to the Profiler.
		/// </summary>
		/// <param name="moduleID">ModuleID that's been updated. Must be the same
		/// as the one we received from the Profiler.</param>
		/// <param name="tmpModulePath">Temporary path to the compiled executable.</param>
		public void ModuleUpdatedCallback(uint moduleID, string tmpModulePath, 
			int nTicksStart, int nTicksWeaveEnd, int nTicksCompileEnd)
		{
			ModuleAndPath moduleAndPath = new ModuleAndPath();
			moduleAndPath.moduleID				= moduleID;
			moduleAndPath.modulePath			= tmpModulePath;
			moduleAndPath.startTime				= nTicksStart;
			moduleAndPath.weaveTimeEnd			= nTicksWeaveEnd;
			moduleAndPath.compileTimeEnd		= nTicksCompileEnd;

			MyTrace.Log(TraceLevel.Info, "Queuing up module to send {0} ", moduleID);

			_moduleSenderWorker.Send(moduleAndPath);
		}
	}
}
