///////////////////////////////////////////////////////////////////////
//  Copyright (C) 2004 Columbia University.  All Rights Reserved.
//
//      $Header: /Goblin/Watcher/FileWatcher.cs 5     5/28/05 1:23p Eaddy $
//
//    File name:	FileWatcher.cs
//   Created By:	Marc Eaddy
//   Created on:	10/10/04
//
//      Purpose:	
//
//  Description:	
//
///////////////////////////////////////////////////////////////////////
/**
 * $Log: /Goblin/Watcher/FileWatcher.cs $
 * 
 * 5     5/28/05 1:23p Eaddy
 * o Changed copyright
 * 
 * 4     5/25/05 6:34p Eaddy
 * o Now uses EnC.Watcher namespace
 * 
 * 3     4/04/05 2:22p Eaddy
 * o Added FileWatcherCollection to simplify watching a collection of
 * files
 * 
 * 2     4/01/05 11:15a Eaddy
 * o Now supports rudimentary weaving
 * Revision 1.6  2005/01/14 23:33:29  me133
 * o Now always recompiles the first time a file is changed
 *
 * Revision 1.5  2004/10/17 08:44:22  me133
 * o Changed member variable names
 *
 * Revision 1.4  2004/10/11 05:57:54  me133
 * no message
 *
 * Revision 1.3  2004/10/10 07:40:07  me133
 * o Now outputs trace messages to log file usnig System.Diagnostics.Trace
 *
 * Revision 1.2  2004/10/10 06:07:24  me133
 * o Added header comment
 */

using System;
using System.IO;
using System.Collections;
using System.Diagnostics;
using System.Security.Cryptography;
using System.Runtime.InteropServices;

namespace EnC.Watcher
{
	public class MyFileWatcher
	{
		public delegate void OnChangedDelegate(string fullPath);

		// Constants

		const uint COR_E_IO					= 0x80131620;
		const uint ERROR_SHARING_VIOLATION	= 32;

		// DATA

		byte[]								_hashCode;
		FileSystemWatcher					_watcher; // IDisposable!
		MD5CryptoServiceProvider			_md5;
		public OnChangedDelegate			_Changed;
		public string						_path;
		static bool							s_firstChange = true;

		public MyFileWatcher(string file, OnChangedDelegate handlerOnChanged)
		{
			if (handlerOnChanged != null)
			{
				_Changed = handlerOnChanged;
			}

			Debug.Assert(file.Length > 0);

			if (Path.IsPathRooted(file))
			{
				_path = file;
			}
			else
			{
				_path = Directory.GetCurrentDirectory() + 
					Path.DirectorySeparatorChar + 
					file;
			}

			_hashCode = getHashCode(_path);
			watch(_path);
		}

		public MyFileWatcher(string file)
			: this(file, null)
		{ }

		private void watch(string sourcePath)
		{
			_watcher = new FileSystemWatcher();

			_watcher.Path					= Path.GetDirectoryName(sourcePath);
			_watcher.Filter					= Path.GetFileName(sourcePath);
			_watcher.NotifyFilter			= NotifyFilters.LastWrite;
			_watcher.IncludeSubdirectories	= false;

			_watcher.Changed += new FileSystemEventHandler(OnFileChanged);

			_watcher.EnableRaisingEvents = true;
		}

		private void OnFileChanged(object source, FileSystemEventArgs e)
		{
			if (_Changed == null)
				return; // Should I also assert?

			byte[] newHashCode = getHashCode(e.FullPath);

			bool bChanged = false;

			if (s_firstChange)
			{
				bChanged = true;
				s_firstChange = false;
			}
			else if (newHashCode == null || _hashCode == null ||
					newHashCode.Length != _hashCode.Length)
			{
				bChanged = true;
			}
			else
			{
				for(int i = 0; i < newHashCode.Length; ++i)
				{
					if (newHashCode[i] != _hashCode[i])
					{
						bChanged = true;
						break;
					}
				}
			}

			if (bChanged)
			{
				_hashCode = newHashCode;
				_Changed(e.FullPath);
			}
		}

		private byte[] getHashCode(string sourcePath)
		{
			byte[] result = null;

			if (_md5 == null)
			{
				// Lazy instantiation
				_md5 = new MD5CryptoServiceProvider();
			}

			int numRetries = 3;

			while (result == null && numRetries > 0)
			{
				try
				{
					FileStream stream = File.OpenRead(sourcePath);
					using(stream)
					{
						result = _md5.ComputeHash(stream);
					}
				}
				catch(IOException ex)
				{
					// I just want to retry sharing violation exceptions.  However,
					// there is no SharingViolationException.  The system just throws
					// a generic IOException which is also used to indicate other
					// exceptions like file not found.  The error information is not
					// properly propagated up the call stack.  The problem with my
					// solution is that the last Win32 error may have changed from
					// the time the error was generated.
					if ((uint) Marshal.GetLastWin32Error()   != ERROR_SHARING_VIOLATION ||
						(uint) Marshal.GetHRForException(ex) != COR_E_IO)
					{
						// Real exception occurred, report and return
						MyTrace.Log(TraceLevel.Error, ex.ToString());
						return null;
					}
				}
				catch(Exception ex)
				{
					MyTrace.Log(TraceLevel.Error, ex.ToString());
					return null;
				}

				--numRetries;
			}

			return result;
		}

		[STAThread]
		static void Main(string[] args)
		{
			if (args.Length < 1)
			{
				Console.WriteLine("Usage: {0} <source file path>",
					"Watcher");
				return;
			}

			MyFileWatcher host = new MyFileWatcher(args[0]);

			// Wait for the user to quit the program.
			Console.WriteLine("Press \'q\' to quit the sample.");
			while(Console.Read()!='q');
		}
	}

	public class FileWatcherCollection
	{
		/// <summary>
		/// Delegate that MyFileWatcher calls when a file has changed. We cache this for efficiency.
		/// </summary>
		readonly MyFileWatcher.OnChangedDelegate _onFileChangedDelegate_ForMyself;

		/// <summary>
		/// Array of MyFileWatchers. One for each watched aspect file.
		/// </summary>
		ArrayList				_fileWatchers = null;

		public FileWatcherCollection()
		{
			// Initialize the file watcher changed delegate
			_onFileChangedDelegate_ForMyself = new MyFileWatcher.OnChangedDelegate(OnFileChanged);
		}

		/// <summary>
		/// Registers a file to be watched.
		/// </summary>
		/// <param name="path">Path to file.</param>
		public void registerForChanges(string filename)
		{
			lock(this)
			{
				if (_fileWatchers == null)
				{
					_fileWatchers = new ArrayList();
				}
			}

			lock(_fileWatchers)
			{
				_fileWatchers.Add(new MyFileWatcher(filename, _onFileChangedDelegate_ForMyself)); 
			}

			MyTrace.Log(TraceLevel.Info, "Watching file: " + filename);
		}

		/// <summary>
		/// Unregisters a file so its not watched anymore.
		/// </summary>
		/// <param name="filename"></param>
		public void unregisterForChanges(string filename)
		{
			lock(this)
			{
				if (_fileWatchers == null)
				{
					return;
				}
			}

			MyTrace.Log(TraceLevel.Info, "Unwatching file: " + filename);

			lock(_fileWatchers)
			{
				for(int i = 0; i < _fileWatchers.Count; ++i)
				{
					MyFileWatcher watcher = (MyFileWatcher) _fileWatchers[i];
					if (Path.GetFileName(watcher._path) == filename)
					{
						_fileWatchers.RemoveAt(i);
						return;
					}
				}
			}

			MyTrace.Log(TraceLevel.Warning, "Failed to unregister '{0}'. Not found.",
				filename);
		}

		/// <summary>
		/// Internal callback called by MyFileWatcher when a file is changed.
		/// </summary>
		/// <param name="path">Path of changed file.</param>
		protected virtual void OnFileChanged(string path)
		{
			MyTrace.Log(TraceLevel.Info, "Change detected: " + path + ".");
		}
	}
}